Package: easytable
Type: Package
Title: Create Multi-Format Regression Tables
Version: 2.1.1
Authors@R: person("Alfredo", "Hernandez Sanchez",
    email = "alhdzsz@gmail.com",
    role = c("aut", "cre"))
Description: Create publication-ready regression tables in multiple
    formats, including 'Word', 'HTML', 'LaTeX', and 'PDF', from
    statistical models. Supports lm() and glm() models. Includes options for
    marginal effects, control variable grouping, and robust standard errors
    using methods described in Zeileis (2004)
    <doi:10.18637/jss.v011.i10>. Tables can be exported to 'Word' via
    'flextable' or to 'LaTeX' for 'PDF' output.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: dplyr, flextable, magrittr, stats
Suggests: knitr, kableExtra, lmtest, sandwich, margins, rmarkdown,
        testthat (>= 3.0.0), palmerpenguins
VignetteBuilder: knitr
URL: https://github.com/alfredo-hs/easytable
BugReports: https://github.com/alfredo-hs/easytable/issues
NeedsCompilation: no
Packaged: 2026-05-06 20:36:28 UTC; alhdzsz
Author: Alfredo Hernandez Sanchez [aut, cre]
Maintainer: Alfredo Hernandez Sanchez <alhdzsz@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-12 19:00:07 UTC
Built: R 4.5.2; ; 2026-05-12 19:30:03 UTC; unix

# Generated by roxygen2: do not edit by hand

export("%>%")
export(easytable)
export(format_latex)
export(format_word)
importFrom(dplyr,.data)
importFrom(dplyr,across)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,where)
importFrom(magrittr,"%>%")
importFrom(stats,nobs)
importFrom(utils,write.csv)

# easytable <img src="man/figures/logo.png" align="right" width="190"/>

Create regression tables that are easy to use and easy to read.

`easytable` is a workhorse table package for `lm()` and `glm()` models with predictable defaults across Word/HTML and LaTeX/PDF outputs.

[![License: MIT](https://img.shields.io/badge/license-MIT-yellow.svg)](LICENSE)
[![pkgdown](https://img.shields.io/badge/pkgdown-site-blue)](https://alfredo-hs.github.io/easytable/)
[![DOI](https://img.shields.io/badge/DOI-10.5281%2Fzenodo.12345678-blue)](https://doi.org/10.5281/zenodo.18673550)
[![easytable status badge](https://alfredo-hs.r-universe.dev/easytable/badges/version)](https://alfredo-hs.r-universe.dev/easytable)
[![R-CMD-check](https://github.com/alfredo-hs/easytable/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/alfredo-hs/easytable/actions/workflows/R-CMD-check.yaml)

## Why easytable

- One main function: `easytable()`
- Code that is easy to use and tables that are easy to read
- Coherent output style across formats
- Optional export to `.docx` and `.csv`
- Control-variable indicators (like Stata)

## Install

To install `easytable` please run the command below:

```r
install.packages("easytable")
```

## Quick Start (Penguins)

```r
library(easytable)
library(palmerpenguins)

m1 <- lm(body_mass_g ~ flipper_length_mm, data = penguins)
m2 <- lm(body_mass_g ~ flipper_length_mm + species, data = penguins)
m3 <- lm(body_mass_g ~ flipper_length_mm + species + island, data = penguins)

# Default output is Word/flextable (also prints in HTML contexts)
easytable(m1, m2, m3)
```

## Core Usage Patterns

### 1) Basic table

```r
easytable(m1, m2, m3)
```

### 2) Custom model names

```r
easytable(
  m1, m2, m3,
  model.names = c("Baseline", "With Species", "Full Model")
)
```

### 3) Control indicators

```r
easytable(
  m1, m2, m3,
  control.var = c("species", "island")
)
```

### 4) Highlight significant coefficients

```r
easytable(
  m1, m2, m3,
  highlight = TRUE
)
```

### 5) LaTeX output

```r
easytable(
  m1, m2, m3,
  output = "latex"
)
```

### 6) Export files

```r
easytable(
  m1, m2, m3,
  export.word = "mytable.docx",
  export.csv = "mytable.csv"
)
```

## Advanced Options

### Robust standard errors

```r
easytable(m1, m2, robust.se = TRUE)
```

### Marginal effects

```r
easytable(m1, m2, margins = TRUE)
```

### Robust SE + marginal effects

```r
easytable(m1, m2, robust.se = TRUE, margins = TRUE)
```

## Supported Model Classes

The `easytable` supports the following models:

- `lm()`
- `glm()`

## Design Invariants

`easytable` enforces these defaults:

1. Coefficient and SE share one cell with a real line break.
2. Zebra striping applies only to coefficient rows.
3. No per-coefficient horizontal rules.
4. Exactly one divider between coefficient rows and model-stat rows.
5. Control indicators belong to the model-stat block.

See `DESIGN_PHILOSOPHY.md` for the full contributor policy.

## Documentation

- Package site: <https://alfredo-hs.github.io/easytable/>
- Function help: `?easytable`
- Tutorial article: `vignette("penguins-tutorial", package = "easytable")`
- Developer roadmap: `vignette("developer-roadmap", package = "easytable")`
- Agent handoff notes: `AI_NOTES.md`
- Testing protocol: `tests/README.md`
- pkgdown source is defined in `_pkgdown.yml`, `pkgdown/index.md`, and `vignettes/`.

## Citation

```text
Hernandez Sanchez, A. (2026). easytable: Create Multi-Format Regression Tables. Version 2.1.1. https://doi.org/10.5281/zenodo.18673550
```

```text
@misc{easytable2026,
	title = {{e}asytable},
	subtitle = {Create Multi-Format Regression Tables},
	author = {Hernandez Sanchez, Alfredo},
	note = {Version 2.1.1},
	year = {2026},
	month = {02},
	date = {2026-02-17},
	publisher = {Zenodo},
	doi = {10.5281/ZENODO.18673550},
	url = {https://github.com/alfredo-hs/easytable}
}
```

## Acknowledgements

This package was created as an education technology to facilitate statistics teaching. Many thanks to the students at Vilnius University and the University of Bucaramanga for their feedback on usability and design. The development of this package was assisted by AI coding tools such as Gemini `3.1 Pro`, Claude `4.5 Sonnet`, and ChatGPT `5.3 Codex` for code debugging, documentation updates, and package restructuring. 

# Testing Protocol

This directory contains committed, deterministic package tests.

## Scope

- Main test suite: `tests/testthat/`
- Goal: stable CI behavior and reproducible validation

## How to Run Tests

### Core package tests

```r
devtools::test()
```

### In constrained environments (for example, headless AI sessions)

Word rendering tests can be skipped:

```sh
EASYTABLE_SKIP_WORD_TESTS=true Rscript -e "devtools::test()"
```

### Optional helper script

```sh
Rscript tests/run-tests.R core
```

## What Must Stay Stable

- Coefficient cell format: two lines (estimate + stars, then `(SE)`).
- Zebra striping only in the coefficient block.
- One divider line between coefficient block and model-stat block.
- Control indicators (`control.var`) are model-stat rows, not coefficient rows.

# Declare global variables used in dplyr/tidy evaluation to satisfy R CMD check
# These are used in NSE contexts within dplyr pipelines

utils::globalVariables(c(
  "term",
  "estimate",
  "std.error",
  "p.value",
  "significance"
))

test_df <- mtcars

# Ensure the dataset name referenced in model calls exists during tests
mtcars <- mtcars

# Linear models
test_m1 <- stats::lm(mpg ~ wt, data = mtcars)
test_m2 <- stats::lm(mpg ~ wt + hp, data = mtcars)
test_m3 <- stats::lm(mpg ~ wt + hp + am, data = mtcars)

test_models_lm <- list(
  Model1 = test_m1,
  Model2 = test_m2,
  Model3 = test_m3
)

# GLM models (logistic regression)
test_g1 <- stats::glm(am ~ wt, data = mtcars, family = stats::binomial)
test_g2 <- stats::glm(am ~ wt + hp, data = mtcars, family = stats::binomial)

test_models_glm <- list(
  Logit1 = test_g1,
  Logit2 = test_g2
)

# Single model for simple tests
test_single_model <- list(SimpleModel = test_m1)

# Models with control variables
test_models_with_controls <- list(
  Base = stats::lm(mpg ~ wt, data = mtcars),
  Full = stats::lm(mpg ~ wt + hp + am + disp, data = mtcars)
)

#' Collapse control variable rows
#'
#' Collapses multiple rows for control variables (including factor levels and
#' transformations) into a single row per control variable.
#'
#' @param table A data frame with regression results
#' @param control.var Character vector of control variable names to collapse
#'
#' @return A data frame with collapsed control variables
#' @keywords internal
collapse_control_vars <- function(table, control.var, levels_map = NULL) {
  if (is.null(control.var) || length(control.var) == 0) {
    return(table)
  }

  result <- table

  for (var in control.var) {

    # Pattern 1: factor(var)... (e.g., factor(species)Chinstrap)
    pattern_factor <- sprintf("^factor\\(%s\\).*", var)
    result$term <- gsub(pattern_factor, var, result$term, perl = TRUE)

    # Pattern 2: log(var) (e.g., log(income))
    pattern_log <- sprintf("^log\\(%s\\)$", var)
    result$term <- gsub(pattern_log, var, result$term, perl = TRUE)

    # Pattern 3: Variable name directly followed by a factor level.
    # When levels_map is available, match each known level exactly to avoid
    # false positives from variable names that are prefixes of others (e.g.
    # collapsing "hp" must not affect "hpq"). Without levels_map, fall back
    # to the original uppercase-start heuristic.
    if (!is.null(levels_map) && !is.null(levels_map[[var]])) {
      for (level in levels_map[[var]]) {
        pat <- sprintf("^%s%s($|[^[:alnum:]_].*)", var, level)
        result$term <- gsub(pat, var, result$term, perl = TRUE)
      }
    } else {
      pattern_level <- sprintf("^%s[A-Z].*", var)
      result$term <- gsub(pattern_level, var, result$term, perl = TRUE)
    }

    # Pattern 4: Variable name with transformations or interactions (.L, :term, etc.)
    pattern_transform <- sprintf("^%s[^[:alnum:]_].*", var)
    result$term <- gsub(pattern_transform, var, result$term, perl = TRUE)
  }

  result
}

#' Mark control variables with 'Y' indicator
#'
#' Replaces coefficient values with "Y" for control variables to indicate
#' their presence in the model without showing individual coefficients.
#'
#' @param table A data frame with regression results
#' @param control.var Character vector of control variable names
#'
#' @return A data frame with control variables marked as "Y"
#' @keywords internal
mark_control_vars <- function(table, control.var) {
  if (is.null(control.var) || length(control.var) == 0) {
    return(table)
  }

  for (col in names(table)[-1]) {
    # TRUE only when this row is a control var AND the cell has a value
    replace_indices <- (table$term %in% control.var) & !is.na(table[[col]]) & (table[[col]] != "")
    table[replace_indices, col] <- "Y"
  }

  table
}

#' Remove duplicate control variable rows
#'
#' After collapsing control variables, removes duplicate rows but keeps
#' one representative row per control variable for each model.
#'
#' @param table A data frame with regression results
#'
#' @return A data frame with duplicates removed
#' @keywords internal
deduplicate_control_vars <- function(table) {

  result <- table

  for (col in names(result)[-1]) {
    duplicate_mask <- duplicated(result$term) & result[[col]] == "Y"
    duplicate_mask[is.na(duplicate_mask)] <- FALSE
    result <- result[!duplicate_mask, , drop = FALSE]
  }

  result
}

#' Sort table with control variables last
#'
#' Sorts the table so that control variables (marked with "Y") appear
#' after regular variables, ordered by how many models they appear in.
#'
#' @param table A data frame with regression results
#'
#' @return A sorted data frame
#' @keywords internal
sort_table <- function(table) {
  y_count <- apply(table[, -1, drop = FALSE], 1, function(row) sum(row == "Y"))
  table[order(y_count, decreasing = FALSE), , drop = FALSE]
}

#' Separate model statistics from coefficients
#'
#' Splits the table into coefficient rows and model-stat rows (control indicators,
#' N, R sq., etc.), removes empty stat rows, then recombines with stats at the bottom.
#'
#' @param table A data frame with regression results
#' @param control.var Character vector of control variable names
#'
#' @return A data frame with measures at the bottom
#' @keywords internal
separate_measures <- function(table, control.var = NULL) {
  measure_names <- get_measure_names()
  control_names <- if (is.null(control.var)) character(0) else control.var
  stat_terms <- unique(c(control_names, measure_names))

  stats <- table %>% dplyr::filter(term %in% stat_terms)

  # Remove stat rows with no data (all empty except term)
  stats <- stats[rowSums(stats[-1] != "") > 0, , drop = FALSE]
  if (nrow(stats) > 0) {
    stats_order <- match(stats$term, stat_terms)
    stats <- stats[order(stats_order), , drop = FALSE]
  }

  coefficients <- table %>% dplyr::filter(!term %in% stat_terms)

  result <- dplyr::bind_rows(coefficients, stats)
  attr(result, "stat_terms") <- stat_terms
  result
}

#' Transform parsed model table
#'
#' Main transformation function that handles control variables, sorting,
#' deduplication, organization, and term label formatting.
#'
#' @param parsed_table A data frame from parse_models()
#' @param control.var Character vector of control variable names
#' @param abbreviate Logical. Abbreviate variable names? Default FALSE
#'
#' @return A transformed data frame ready for formatting
#' @keywords internal
transform_table <- function(parsed_table, control.var = NULL, abbreviate = FALSE) {

  result <- parsed_table

  # Pull factor level map carried from parse_models()
  levels_map <- attr(parsed_table, "levels_map")

  # Handle control variables if specified
  if (!is.null(control.var)) {
    result <- collapse_control_vars(result, control.var, levels_map)
    result <- mark_control_vars(result, control.var)
    result <- deduplicate_control_vars(result)
  }

  # Replace NA with empty string
  result[is.na(result)] <- ""

  # Drop rows that are fully empty artifacts (empty term and no model values)
  is_empty_row <- (result$term == "") &
    (rowSums(result[, -1, drop = FALSE] != "") == 0)
  result <- result[!is_empty_row, , drop = FALSE]

  # Sort table (control vars last)
  result <- sort_table(result)

  # Separate model-stat rows and put them at the bottom
  result <- separate_measures(result, control.var)

  stat_terms <- attr(result, "stat_terms")
  if (is.null(stat_terms) || length(stat_terms) == 0) {
    stat_terms <- get_measure_names()
  }

  # Add row metadata to remove brittle string inference downstream
  result$row_type <- "coefficient"
  result$row_type[result$term == "(Intercept)"] <- "intercept"
  
  result$row_type[result$term %in% stat_terms] <- "statistic"

  if (!is.null(control.var)) {
    result$row_type[result$term %in% control.var] <- "control"
  }

  # Format term labels for display (skip model-stat rows)
  non_measure_rows <- !result$term %in% stat_terms

  result$term[non_measure_rows] <- format_term_labels(
    result$term[non_measure_rows],
    row_types = result$row_type[non_measure_rows],
    levels_map = levels_map,
    abbreviate = abbreviate
  )

  attr(result, "stat_terms") <- stat_terms

  result
}

test_that("easytable works with Word output (default)", {
  skip_if_word_tests_unavailable()

  result <- easytable(test_m1)

  expect_s3_class(result, "flextable")
})

test_that("easytable rejects markdown output", {
  expect_error(
    easytable(test_m1, output = "markdown"),
    "Unknown output format"
  )
})

test_that("easytable works with latex output", {
  skip_if_not_installed("knitr")

  result <- easytable(test_m1, output = "latex")

  expect_type(result, "character")
  expect_true(nchar(result) > 0)
})

test_that("easytable handles multiple lm models", {
  skip_if_word_tests_unavailable()

  result <- easytable(test_m1, test_m2, test_m3, output = "word")

  expect_s3_class(result, "flextable")
})

test_that("easytable handles glm models", {
  skip_if_word_tests_unavailable()

  result <- easytable(test_g1, test_g2, output = "word")

  expect_s3_class(result, "flextable")
})

test_that("easytable with robust standard errors", {
  skip_if_word_tests_unavailable()
  skip_if_not_installed("lmtest")
  skip_if_not_installed("sandwich")

  result <- easytable(test_m1, output = "word", robust.se = TRUE)

  expect_s3_class(result, "flextable")
})

test_that("easytable with marginal effects", {
  skip_if_word_tests_unavailable()
  skip_if_not_installed("margins")

  result <- easytable(test_m1, output = "word", margins = TRUE)

  expect_s3_class(result, "flextable")
})

test_that("easytable with robust SE and margins", {
  skip_if_word_tests_unavailable()
  skip_if_not_installed("lmtest")
  skip_if_not_installed("sandwich")
  skip_if_not_installed("margins")

  result <- easytable(test_m2, test_m3, output = "word", robust.se = TRUE, margins = TRUE)

  expect_s3_class(result, "flextable")
})

test_that("easytable with control variables", {
  skip_if_word_tests_unavailable()

  result <- easytable(
    test_m1, test_m2, test_m3,
    output = "word",
    control.var = c("hp", "am")
  )

  expect_s3_class(result, "flextable")
})

test_that("easytable full pipeline with all features", {
  skip_if_word_tests_unavailable()
  skip_if_not_installed("lmtest")
  skip_if_not_installed("sandwich")

  temp_csv <- tempfile()

  result <- easytable(
    test_m1, test_m2, test_m3,
    output = "word",
    export.csv = paste0(temp_csv, ".csv"),
    robust.se = TRUE,
    control.var = c("hp", "am"),
    highlight = TRUE
  )

  expect_s3_class(result, "flextable")
  expect_true(file.exists(paste0(temp_csv, ".csv")))

  unlink(paste0(temp_csv, ".csv"))
})

test_that("export.word creates a non-empty .docx file", {
  skip_if_word_tests_unavailable()

  tmp <- tempfile(fileext = ".docx")
  on.exit(unlink(tmp), add = TRUE)

  result <- easytable(test_m1, test_m2, export.word = tmp)

  expect_s3_class(result, "flextable")
  expect_true(file.exists(tmp))
  expect_true(file.info(tmp)$size > 0)
})

test_that("easytable latex pipeline with control vars", {
  skip_if_not_installed("knitr")

  result <- easytable(
    test_m1, test_m2, test_m3,
    output = "latex",
    control.var = c("hp", "am")
  )

  expect_type(result, "character")
  expect_true(grepl("hp", result))
  expect_true(grepl("am", result))
})

test_that("penguins dataset end-to-end test", {
  skip_if_word_tests_unavailable()
  skip_if_not_installed("palmerpenguins")

  penguins <- palmerpenguins::penguins

  m1 <- lm(body_mass_g ~ flipper_length_mm, data = penguins)
  m2 <- lm(body_mass_g ~ flipper_length_mm + species, data = penguins)
  m3 <- lm(body_mass_g ~ flipper_length_mm + species + island, data = penguins)

  result1 <- easytable(m1, m2, m3)
  expect_s3_class(result1, "flextable")

  result2 <- easytable(m1, m2, m3, control.var = c("species", "island"))
  expect_s3_class(result2, "flextable")

  result3 <- easytable(m1, m2, m3, highlight = TRUE)
  expect_s3_class(result3, "flextable")
})

test_that("custom.row appears in the transformed table below AIC", {
  parsed <- parse_models(test_models_lm)
  transformed <- transform_table(parsed)

  # Simulate what easytable does after transform
  model_cols <- setdiff(names(transformed), c("term", "row_type"))
  new_row <- data.frame(term = "F. Statistic", stringsAsFactors = FALSE)
  vals <- c(".004", ".3", ".1")
  for (i in seq_along(model_cols)) {
    new_row[[model_cols[i]]] <- vals[i]
  }
  new_row$row_type <- "custom"
  result <- rbind(transformed, new_row)

  # Custom row label is present
  expect_true("F. Statistic" %in% result$term)

  # Custom row appears after the last stat row (lm models have N, R sq., Adj. R sq.)
  last_stat_idx <- max(which(result$term %in% c("N", "R sq.", "Adj. R sq.", "AIC")))
  custom_idx <- which(result$term == "F. Statistic")
  expect_true(custom_idx > last_stat_idx)
})

test_that("easytable custom.row word output includes custom row", {
  skip_if_word_tests_unavailable()

  result <- easytable(test_m1, test_m2,
                      custom.row = c("F. Statistic", ".004", ".3"))

  expect_s3_class(result, "flextable")
  # flextable body should contain the custom label
  ft_data <- result$body$dataset
  expect_true("F. Statistic" %in% ft_data$term)
})

test_that("easytable custom.row latex output includes custom row", {
  skip_if_not_installed("knitr")

  result <- easytable(test_m1, test_m2,
                      output = "latex",
                      custom.row = c("F. Statistic", ".004", ".3"))

  expect_true(grepl("F. Statistic", as.character(result), fixed = TRUE))
})

test_that("easytable errors with wrong custom.row length", {
  expect_error(
    easytable(test_m1, test_m2, custom.row = c("F. Statistic", ".004")),
    "must have 3 elements"
  )
})

test_that("collapse_control_vars handles factor variables", {
  test_table <- data.frame(
    term = c("(Intercept)", "x1", "factor(species)Chinstrap", "factor(species)Gentoo"),
    Model1 = c("1.0", "2.0", "3.0", "4.0")
  )

  result <- collapse_control_vars(test_table, "species")

  # Both factor levels should be collapsed to "species"
  expect_true("species" %in% result$term)
  expect_false("factor(species)Chinstrap" %in% result$term)
  expect_false("factor(species)Gentoo" %in% result$term)
})

test_that("collapse_control_vars handles log transformations", {
  test_table <- data.frame(
    term = c("(Intercept)", "x1", "log(income)"),
    Model1 = c("1.0", "2.0", "3.0")
  )

  result <- collapse_control_vars(test_table, "income")

  expect_true("income" %in% result$term)
  expect_false("log(income)" %in% result$term)
})

test_that("collapse_control_vars handles lowercase factor levels via levels_map", {
  # Regression test: factor levels starting with lowercase were not matched by
  # the original ^var[A-Z].* pattern (e.g. gendermale, genderfemale)
  test_table <- data.frame(
    term = c("(Intercept)", "age", "gendermale", "genderfemale"),
    Model1 = c("0.5", "1.0", "2.0", "3.0"),
    stringsAsFactors = FALSE
  )
  levels_map <- list(gender = c("female", "male"))

  result <- collapse_control_vars(test_table, "gender", levels_map)

  expect_true("gender" %in% result$term)
  expect_false("gendermale" %in% result$term)
  expect_false("genderfemale" %in% result$term)
  expect_true("age" %in% result$term)  # unrelated term unchanged
})

test_that("collapse_control_vars with levels_map does not affect prefix-named terms", {
  # "hp" as control var must not collapse "hpq" even when levels_map is empty for hp
  test_table <- data.frame(
    term = c("hp", "hpq", "hp2"),
    Model1 = c("1.0", "2.0", "3.0"),
    stringsAsFactors = FALSE
  )
  levels_map <- list()  # hp not a factor, no levels

  result <- collapse_control_vars(test_table, "hp", levels_map)

  expect_true("hpq" %in% result$term)
  expect_true("hp2" %in% result$term)
})

test_that("collapse_control_vars avoids greedy matching", {
  # Test the critical fix: "hp" should not match "hpq"
  test_table <- data.frame(
    term = c("hp", "hpq", "hp2"),
    Model1 = c("1.0", "2.0", "3.0")
  )

  result <- collapse_control_vars(test_table, "hp")

  # Only "hp" should be affected, not "hpq"
  expect_true("hp" %in% result$term)
  expect_true("hpq" %in% result$term)  # Should remain unchanged
})

test_that("collapse_control_vars handles NULL control.var", {
  test_table <- data.frame(
    term = c("(Intercept)", "x1"),
    Model1 = c("1.0", "2.0")
  )

  result <- collapse_control_vars(test_table, NULL)

  # Should return unchanged
  expect_equal(result, test_table)
})

test_that("mark_control_vars replaces values with Y", {
  test_table <- data.frame(
    term = c("x1", "species", "island"),
    Model1 = c("1.0", "2.0", "3.0"),
    Model2 = c("1.5", "2.5", NA)
  )

  result <- mark_control_vars(test_table, c("species", "island"))

  expect_equal(result$Model1[2], "Y")
  expect_equal(result$Model1[3], "Y")
  expect_equal(result$Model2[2], "Y")
  expect_true(is.na(result$Model2[3])) # NA should remain NA
  expect_equal(result$Model1[1], "1.0") # Non-control var unchanged
})

test_that("deduplicate_control_vars removes duplicate Y rows", {
  test_table <- data.frame(
    term = c("x1", "species", "species", "island"),
    Model1 = c("1.0", "Y", "Y", "3.0"),
    Model2 = c("1.5", "Y", "Y", "")
  )

  result <- deduplicate_control_vars(test_table)

  # Should have only one "species" row
  expect_equal(sum(result$term == "species"), 1)
})

test_that("deduplicate_control_vars does not create NA artifact rows", {
  test_table <- data.frame(
    term = c("island", "island"),
    Model1 = c(NA, NA),
    Model2 = c(NA, NA),
    Model3 = c("Y", "Y")
  )

  result <- deduplicate_control_vars(test_table)

  expect_equal(nrow(result), 1)
  expect_false(any(is.na(result$term)))
})

test_that("sort_table puts control variables last", {
  test_table <- data.frame(
    term = c("x1", "species", "island", "x2"),
    Model1 = c("1.0", "Y", "Y", "2.0"),
    Model2 = c("1.5", "Y", "", "2.5")
  )

  result <- sort_table(test_table)

  # Regular variables should come before control variables
  y_positions <- which(result$Model1 == "Y" | result$Model2 == "Y")
  regular_positions <- which(!(result$Model1 == "Y" | result$Model2 == "Y"))

  if (length(y_positions) > 0 && length(regular_positions) > 0) {
    expect_true(min(y_positions) > max(regular_positions))
  }
})

test_that("separate_measures puts measures at bottom", {
  test_table <- data.frame(
    term = c("x1", "N", "x2", "R sq.", "AIC"),
    Model1 = c("1.0", "100", "2.0", "0.5", "200")
  )

  result <- separate_measures(test_table)

  # Measures should be at the bottom
  measure_names <- c("N", "R sq.", "Adj. R sq.", "AIC")
  measure_rows <- which(result$term %in% measure_names)

  if (length(measure_rows) > 0) {
    # All measure rows should be consecutive at the end
    expect_equal(measure_rows, seq(from = nrow(result) - length(measure_rows) + 1, to = nrow(result)))
  }
})

test_that("separate_measures removes empty measure rows", {
  test_table <- data.frame(
    term = c("x1", "N", "R sq.", "AIC"),
    Model1 = c("1.0", "100", "", ""),
    Model2 = c("2.0", "200", "0.5", "")
  )

  result <- separate_measures(test_table)

  # Empty AIC row should be removed (all empty except term)
  # But we keep rows that have at least one non-empty value
  expect_true(nrow(result) <= nrow(test_table))
})

test_that("transform_table integrates all transformations", {
  parsed <- parse_models(test_models_lm, robust.se = FALSE, margins = FALSE)
  result <- transform_table(parsed, control.var = c("species", "island"))

  expect_s3_class(result, "data.frame")
  expect_true(all(!is.na(result))) # No NA values should remain

  # Control variables should be marked with Y
  species_row <- result[result$term == "species", ]
  if (nrow(species_row) > 0) {
    expect_true(any(species_row[-1] == "Y"))
  }

  # Measures should be at bottom
  measure_names <- c("N", "R sq.", "Adj. R sq.", "AIC")
  last_rows <- tail(result$term, 4)
  expect_true(any(last_rows %in% measure_names))
})

test_that("transform_table handles NULL control.var", {
  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  result <- transform_table(parsed, control.var = NULL)

  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)
})

test_that("row_type metadata is created during transformation", {
  m1 <- lm(mpg ~ cyl + hp, data = mtcars)
  parsed <- parse_models(list("Model 1" = m1))
  transformed <- transform_table(parsed)
  
  expect_true("row_type" %in% names(transformed))
  expect_true(all(transformed$row_type %in% c("intercept", "coefficient", "statistic")))
  
  # Check intercept specifically
  intercept_idx <- which(transformed$term == "(Intercept)")
  expect_equal(transformed$row_type[intercept_idx], "intercept")
})

test_that("row_type handles controls and custom rows properly", {
  m1 <- lm(mpg ~ cyl + hp + wt, data = mtcars)
  m2 <- lm(mpg ~ cyl + hp + wt + am, data = mtcars)
  
  parsed <- parse_models(list("M1" = m1, "M2" = m2))
  transformed <- transform_table(parsed, control.var = c("wt", "am"))
  
  expect_true(all(transformed$row_type[transformed$term %in% c("wt", "am")] == "control"))
})

test_that("row_type is stripped from final word and latex output", {
  m1 <- lm(mpg ~ cyl, data = mtcars)
  
  # Word parsing
  out_word <- suppressWarnings(easytable(m1, output = "word"))
  # flextable's internal body dataset should NOT have row_type
  expect_false("row_type" %in% out_word$col_keys)
  
  # LaTeX
  out_latex <- as.character(suppressWarnings(easytable(m1, output = "latex")))
  # Check latex string has no reference to row_type (just an extra safety)
  expect_false(grepl("row_type", out_latex, fixed = TRUE))
})

test_that("formatting uses row_type instead of literal 'Y' for control rows", {
  # Mock a variable named "Y" to prove we don't accidentally stripe it incorrectly
  # by confusing it with the literal control indicator "Y"
  mtcars_mock <- mtcars
  mtcars_mock$Y <- mtcars_mock$wt
  
  m1 <- lm(mpg ~ cyl + Y, data = mtcars_mock)
  
  # If the logic mistakenly matched the coefficient value "Y" or term "Y",
  # it might act strangely.
  res_word <- suppressWarnings(easytable(m1, output = "word"))
  res_latex <- as.character(suppressWarnings(easytable(m1, output = "latex")))
  
  # Just verify it rendered successfully without crashing or mutating the name
  expect_true(is.character(res_latex))
})

test_that("format_word creates flextable object", {
  skip_if_word_tests_unavailable()

  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_word(transformed, robust.se = FALSE, margins = FALSE, highlight = FALSE)

  expect_s3_class(result, "flextable")
  expect_true(any(grepl("Coefficient", unlist(result), fixed = TRUE)))
})

test_that("format_word includes significance footnote", {
  skip_if_word_tests_unavailable()

  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_word(transformed, robust.se = FALSE, margins = FALSE, highlight = FALSE)

  # Check that flextable has footer (contains significance info)
  expect_s3_class(result, "flextable")
})

test_that("format_word adds robust SE note", {
  skip_if_word_tests_unavailable()
  skip_if_not_installed("lmtest")
  skip_if_not_installed("sandwich")

  parsed <- parse_models(test_single_model, robust.se = TRUE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_word(transformed, robust.se = TRUE, margins = FALSE, highlight = FALSE)

  expect_s3_class(result, "flextable")
})

test_that("format_word adds margins note", {
  skip_if_word_tests_unavailable()
  skip_if_not_installed("margins")

  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = TRUE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_word(transformed, robust.se = FALSE, margins = TRUE, highlight = FALSE)

  expect_s3_class(result, "flextable")
})

test_that("format_word handles highlighting", {
  skip_if_word_tests_unavailable()

  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_word(transformed, robust.se = FALSE, margins = FALSE, highlight = TRUE)

  expect_s3_class(result, "flextable")
})

test_that("format_word uses multiplication sign for interaction display", {
  skip_if_word_tests_unavailable()

  m <- lm(mpg ~ wt * hp, data = mtcars)
  parsed <- parse_models(list(Model1 = m), robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_word(transformed, robust.se = FALSE, margins = FALSE, highlight = FALSE)

  expect_true(any(grepl("×", result$body$dataset$term, fixed = TRUE)))
  expect_false(any(grepl(" \\* ", result$body$dataset$term)))
})

test_that("easytable word output handles scientific notation and long model headers", {
  skip_if_word_tests_unavailable()

  long_model_name <- paste(rep("A", 18), collapse = "")

  set.seed(123)
  df <- data.frame(
    log_gdp = seq(8, 12, length.out = 120),
    pop = seq(1e6, 4e6, length.out = 120)
  )
  df$lifeExp <- 62 +
    1.2e-8 * df$log_gdp -
    3.5e-9 * df$pop +
    7.4e-15 * (df$log_gdp * df$pop) +
    rnorm(nrow(df), sd = 1e-5)

  m1 <- lm(lifeExp ~ log_gdp + pop, data = df)
  m2 <- lm(lifeExp ~ log_gdp * pop, data = df)

  result <- easytable(
    m1, m2,
    digits = 4,
    model.names = c(long_model_name, "b")
  )

  expect_s3_class(result, "flextable")
  expect_true(any(grepl("×", result$body$dataset$term, fixed = TRUE)))
  expect_true(any(grepl("E-", unlist(result$body$dataset[-1]), fixed = TRUE)))
  expect_true(any(grepl(long_model_name, unlist(result), fixed = TRUE)))
})

test_that("format_latex creates character output", {
  skip_if_not_installed("knitr")

  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_latex(transformed, robust.se = FALSE, margins = FALSE, highlight = FALSE)

  expect_type(result, "character")
  expect_true(nchar(result) > 0)
})

test_that("format_latex includes LaTeX table elements", {
  skip_if_not_installed("knitr")

  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_latex(transformed, robust.se = FALSE, margins = FALSE, highlight = FALSE)

  # Should contain LaTeX elements (if using kableExtra, otherwise basic elements)
  expect_true(grepl("\\\\", result) || grepl("tabular", result))
  expect_true(grepl("Coefficient", result, fixed = TRUE))
})

test_that("format_latex includes significance footnote", {
  skip_if_not_installed("knitr")

  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- format_latex(transformed, robust.se = FALSE, margins = FALSE, highlight = FALSE)

  expect_true(grepl("Significance", result))
})

test_that("format_latex uses a LaTeX-safe multiplication sign for interactions", {
  skip_if_not_installed("knitr")

  m <- lm(mpg ~ wt * hp, data = mtcars)
  parsed <- parse_models(list(Model1 = m), robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL)

  result <- as.character(
    format_latex(transformed, robust.se = FALSE, margins = FALSE, highlight = FALSE)
  )

  expect_true(grepl("\\times", result, fixed = TRUE))
})

test_that("format_latex highlights negative scientific coefficients by coefficient sign only", {
  skip_if_not_installed("knitr")
  skip_if_not_installed("kableExtra")

  negative_coef <- data.frame(
    term = "x",
    Model1 = "-3.25E-5 ***\n(1.20E-6)",
    stringsAsFactors = FALSE
  )
  negative_result <- as.character(
    format_latex(negative_coef, robust.se = FALSE, margins = FALSE, highlight = TRUE)
  )
  expect_true(grepl("ffcccc", negative_result, fixed = TRUE))

  positive_coef_negative_se <- data.frame(
    term = "x",
    Model1 = "3.25E-5 ***\n(-1.20E-6)",
    stringsAsFactors = FALSE
  )
  positive_result <- as.character(
    format_latex(positive_coef_negative_se, robust.se = FALSE, margins = FALSE, highlight = TRUE)
  )
  expect_true(grepl("e6ffe6", positive_result, fixed = TRUE))
  expect_false(grepl("ffcccc", positive_result, fixed = TRUE))
})

#' Create Multi-Format Regression Tables
#'
#' Takes model objects as arguments and creates formatted tables for
#' Word or LaTeX/PDF output. Supports robust standard errors,
#' marginal effects, and control variable grouping.
#'
#' @param ... Statistical model objects (lm or glm). Pass models directly
#'   like \code{easytable(m1, m2, m3)}.
#' @param model.names Character vector of custom names for model columns.
#'   If NULL (default), columns are named "Model 1", "Model 2", etc.
#'   Length must match number of models.
#' @param output Character string specifying output format. One of:
#'   \itemize{
#'     \item \code{"word"} - Microsoft Word via flextable (default)
#'     \item \code{"latex"} - LaTeX for PDF output
#'   }
#' @param export.word Character string ending in \code{.docx} for Word file export.
#'   Only supported when \code{output = "word"}. If NULL (default), no file is written.
#' @param export.csv Character string ending in \code{.csv} for CSV export.
#'   If NULL (default), no CSV file is written.
#' @param robust.se Logical. Use robust standard errors (HC type)? Default FALSE.
#'   Requires packages: lmtest, sandwich
#' @param control.var Character vector of variable names to group as "control
#'   variables". These will be collapsed into single rows showing "Y" for
#'   presence instead of individual coefficients. Default NULL.
#' @param margins Logical. Compute average marginal effects (AME)? Default FALSE.
#'   Requires package: margins
#' @param highlight Logical. Highlight significant coefficients (positive in green,
#'   negative in red)? Default FALSE. Works best with Word output.
#' @param abbreviate Logical. Abbreviate variable names for readability? Default FALSE.
#'   When TRUE, long variable names are shortened using deterministic rules.
#' @param table_size Character string specifying LaTeX table size. Only works with
#'   \code{output = "latex"}. Options: "tiny", "small", "normalsize", "scriptsize".
#'   Default "normalsize". Error if used with Word output.
#' @param digits Number of digits after the decimal point for coefficients and
#'   standard errors, including the mantissa in scientific notation. Allowed
#'   values are 0 to 4. Default 2.
#' @param custom.row Optional character vector for an additional row placed at
#'   the bottom of the statistics block. The first element is the row label and
#'   each subsequent element is the value for the corresponding model column.
#'   The vector must have exactly one more element than the number of models.
#'   Default NULL (no extra row).
#'
#' @return
#' Depends on \code{output}:
#' \itemize{
#'   \item \code{"word"} - A flextable object
#'   \item \code{"latex"} - Character string with LaTeX table code
#' }
#'
#' @details
#' The function extracts coefficients, standard errors, and p-values from each
#' model, adds significance stars (*** p<.01, ** p<.05, * p<.1), and includes
#' model fit statistics such as N, R-squared, Adjusted R-squared, and
#' AIC (for \code{glm} models).
#'
#' Control variables can be grouped to show presence/absence rather than
#' individual coefficients for each factor level or transformation.
#'
#' Term labels are automatically formatted for readability:
#' \itemize{
#'   \item Factor levels separated with colon (e.g., \code{advisor_confidence:low})
#'   \item Interactions shown with a multiplication sign (e.g., \code{var1 × var2})
#'   \item Polynomial contrasts as L indices (e.g., \code{var:L1}, \code{var:L2})
#'   \item Long variable names abbreviated for clarity
#' }
#'
#' @section Dependencies:
#' \itemize{
#'   \item Always required: dplyr
#'   \item Word output: flextable
#'   \item LaTeX output: knitr, optionally kableExtra for enhanced formatting
#'   \item Robust SE: lmtest, sandwich
#'   \item Marginal effects: margins
#' }
#'
#' @export
easytable <- function(...,
                      model.names = NULL,
                      output = "word",
                      export.word = NULL,
                      export.csv = NULL,
                      robust.se = FALSE,
                      control.var = NULL,
                      margins = FALSE,
                      highlight = FALSE,
                      abbreviate = FALSE,
                      table_size = "normalsize",
                      digits = 2,
                      custom.row = NULL) {

  # Capture models from dots
  models <- list(...)

  # Validate that we have at least one model
  if (length(models) == 0) {
    stop("No models were provided. Try: easytable(m1, m2, m3)",
         call. = FALSE)
  }

  # Build model names
  if (is.null(model.names)) {
    # Default: "Model 1", "Model 2", etc.
    model_names <- paste("Model", seq_along(models))
  } else {
    # Validate user-provided names
    if (!is.character(model.names)) {
      stop("`model.names` must be a character vector, for example c(\"Model 1\", \"Model 2\").", call. = FALSE)
    }
    if (length(model.names) != length(models)) {
      stop(
        sprintf(
          "`model.names` has %d value(s), but you supplied %d model(s).",
          length(model.names), length(models)
        ),
        call. = FALSE
      )
    }
    model_names <- model.names
  }

  # Create named list
  names(models) <- model_names
  model_list <- models

  # Validate and normalize output parameter
  output <- validate_output_format(output)

  # Validate inputs
  validate_model_list(model_list)
  validate_model_types(model_list)
  validate_control_vars(model_list, control.var)
  validate_parameters(robust.se, margins, highlight, export.word, export.csv, output)
  validate_table_size(table_size, output)
  validate_digits(digits)
  validate_custom_row(custom.row, length(model_list))

  # Check feature dependencies
  check_robust_dependencies(robust.se)
  check_margins_dependencies(margins)
  check_format_dependencies(output)

  # Parse models (extract coefficients, SE, p-values)
  parsed <- parse_models(model_list, robust.se, margins, digits)

  # Transform table (handle control vars, sort, organize)
  transformed <- transform_table(parsed, control.var, abbreviate)

  # Append custom row below AIC if provided
  if (!is.null(custom.row)) {
    # model_cols should exclude row_type
    model_cols <- setdiff(names(transformed), c("term", "row_type"))
    new_row <- data.frame(term = custom.row[1], stringsAsFactors = FALSE)
    for (i in seq_along(model_cols)) {
      new_row[[model_cols[i]]] <- custom.row[i + 1L]
    }
    new_row$row_type <- "custom"

    transformed <- rbind(transformed, new_row)
    existing_stat_terms <- attr(transformed, "stat_terms")
    attr(transformed, "stat_terms") <- c(existing_stat_terms, custom.row[1])
  }

  # Export to CSV if requested
  if (!is.null(export.csv)) {
    csv_table <- transformed
    csv_table$row_type <- NULL
    for (col in names(csv_table)) {
      if (is.character(csv_table[[col]])) {
        csv_table[[col]] <- gsub("\n", " ", csv_table[[col]], fixed = TRUE)
      }
    }
    write.csv(
      csv_table,
      file = export.csv,
      row.names = FALSE
    )
  }

  # Format based on output type
  result <- switch(
    output,
    word = format_word(transformed, robust.se, margins, highlight),
    latex = format_latex(transformed, robust.se, margins, highlight, table_size)
  )

  # Export Word file if requested
  if (!is.null(export.word)) {
    flextable::save_as_docx(`Regression Table` = result, path = export.word)
  }

  return(result)
}

# Test the easytable() interface

test_that("easytable accepts model objects through dots", {
  # Simple models
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)

  # Should work without error
  expect_no_error(
    easytable(m1, m2, output = "latex")
  )
})

test_that("easytable uses default Model N naming", {
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)
  m3 <- lm(mpg ~ wt + hp + qsec, data = mtcars)

  result <- easytable(m1, m2, m3, output = "latex")

  # Check that result contains "Model 1", "Model 2", "Model 3"
  expect_true(grepl("Model 1", result))
  expect_true(grepl("Model 2", result))
  expect_true(grepl("Model 3", result))
})

test_that("easytable respects custom model.names", {
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)

  result <- easytable(m1, m2,
                      model.names = c("Baseline", "Full"),
                      output = "latex")

  # Check that custom names appear in output
  expect_true(grepl("Baseline", result))
  expect_true(grepl("Full", result))

  # Check that default names do NOT appear
  expect_false(grepl("Model 1", result))
  expect_false(grepl("Model 2", result))
})

test_that("easytable validates model.names length", {
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)

  # Should error when length mismatch
  expect_error(
    easytable(m1, m2, model.names = c("Only One")),
    "model.names.*supplied.*model"
  )

  expect_error(
    easytable(m1, m2, model.names = c("One", "Two", "Three")),
    "model.names.*supplied.*model"
  )
})

test_that("easytable ignores named arguments in dots", {
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)

  # Even with named arguments, should use default Model N naming
  result <- easytable(Logit = m1, Probit = m2, output = "latex")

  # Should still use Model 1, Model 2
  expect_true(grepl("Model 1", result))
  expect_true(grepl("Model 2", result))

  # Named arguments should NOT appear
  expect_false(grepl("Logit", result))
  expect_false(grepl("Probit", result))
})

test_that("easytable errors with no models", {
  expect_error(
    easytable(output = "latex"),
    "No models were provided"
  )
})

test_that("easytable works with single model", {
  m1 <- lm(mpg ~ wt, data = mtcars)

  result <- easytable(m1, output = "latex")

  expect_true(grepl("Model 1", result))
})

test_that("easytable works with lm models", {
  m1 <- lm(mpg ~ wt, data = mtcars)
  m2 <- lm(mpg ~ wt + hp, data = mtcars)

  expect_no_error(
    easytable(m1, m2, output = "latex")
  )
})

test_that("easytable works with glm models", {
  # Binary outcome for glm
  mtcars_bin <- mtcars
  mtcars_bin$am_binary <- mtcars_bin$am

  m1 <- glm(am_binary ~ wt, data = mtcars_bin, family = binomial)
  m2 <- glm(am_binary ~ wt + hp, data = mtcars_bin, family = binomial)

  expect_no_error(
    easytable(m1, m2, output = "latex")
  )
})

# Test factor level formatting with synthetic data
test_that("factor terms display as var:level not varlevel", {
  # Create synthetic data with factor that has levels low, mid, high
  set.seed(123)
  test_data <- data.frame(
    y = rnorm(100),
    x = rnorm(100),
    advisor_confidence = factor(rep(c("low", "mid", "high"), length.out = 100))
  )

  m1 <- lm(y ~ x + advisor_confidence, data = test_data)
  result <- easytable(m1, output = "latex")

  # Should contain "adv_conf:low" or "advisor_confidence:low" (depending on abbreviation)
  # but NOT "advisor_confidencelow" (concatenated)
  expect_true(grepl(":low", result))
  expect_true(grepl(":mid", result))

  # Should NOT contain concatenated form
  expect_false(grepl("advisor_confidencelow", result, fixed = TRUE))
  expect_false(grepl("advisor_confidencemid", result, fixed = TRUE))
})

test_that("interaction with contrast and factor displays with a multiplication sign", {
  # Create data with ordered factor (to get polynomial contrasts) and regular factor
  set.seed(123)
  test_data <- data.frame(
    y = rnorm(100),
    financial_prudence = factor(rep(1:4, length.out = 100), ordered = TRUE),
    digital_confidence = factor(rep(c("low", "high"), length.out = 100))
  )

  m1 <- lm(y ~ financial_prudence * digital_confidence, data = test_data)
  result <- easytable(m1, output = "latex")

  # Should have polynomial contrast labels from ordered factors
  expect_true(grepl("financial\\.prudence\\.(L|Q|C)", result))

  # Should have interaction with a multiplication sign in LaTeX-safe form
  expect_true(grepl("\\times", result, fixed = TRUE))

  # Should have factor level separated
  expect_true(grepl(":low|:high", result))
})

#' Pipe Operator
#'
#' Re-export of `%>%` from 'magrittr'.
#'
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @return The result of evaluating the right-hand side expression after the
#'   left-hand side has been passed through the pipe.
#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`

#!/usr/bin/env Rscript

args <- commandArgs(trailingOnly = TRUE)
profile <- if (length(args) >= 1) args[[1]] else "core"

cat("easytable test runner\n")
cat("Profile:", profile, "\n")

# During R CMD check, tests are already executed via tests/testthat.R.
# Skip here to avoid duplicate execution and check-layout path issues.
if (nzchar(Sys.getenv("_R_CHECK_PACKAGE_NAME_"))) {
  cat("Detected R CMD check environment; skipping tests/run-tests.R.\n")
  cat("All requested test layers completed.\n")
  quit(save = "no", status = 0)
}

# Keep runtime deterministic in constrained/headless environments.
Sys.setenv(
  OMP_NUM_THREADS = "1",
  OPENBLAS_NUM_THREADS = "1",
  MKL_NUM_THREADS = "1",
  VECLIB_MAXIMUM_THREADS = "1",
  NUMEXPR_NUM_THREADS = "1"
)

find_package_root <- function(start = getwd()) {
  dir <- normalizePath(start, winslash = "/", mustWork = TRUE)
  initial <- dir

  repeat {
    if (file.exists(file.path(dir, "DESCRIPTION"))) {
      return(dir)
    }

    parent <- dirname(dir)
    if (identical(parent, dir)) {
      break
    }
    dir <- parent
  }

  # R CMD check layout fallback:
  # <tmp>/easytable.Rcheck/tests (wd) and package source at
  # <tmp>/easytable.Rcheck/<pkg>/DESCRIPTION.
  pkg_name <- Sys.getenv("_R_CHECK_PACKAGE_NAME_", "easytable")
  rcheck_root <- dirname(initial)
  sibling_pkg_root <- file.path(rcheck_root, pkg_name)
  if (file.exists(file.path(sibling_pkg_root, "DESCRIPTION"))) {
    return(sibling_pkg_root)
  }

  stop("Could not locate package root from: ", start, call. = FALSE)
}

run_core <- function(pkg_root) {
  if (!requireNamespace("testthat", quietly = TRUE)) {
    stop("Package 'testthat' is required to run tests. Install with install.packages('testthat').")
  }

  testthat::test_local(
    path = pkg_root,
    stop_on_failure = TRUE
  )
}

run_xtest <- function(pkg_root) {
  run_xtest <- tolower(Sys.getenv("EASYTABLE_RUN_XTEST", "false")) %in% c("1", "true", "yes", "y")
  if (!run_xtest) {
    cat("Skipping tests/xtest (set EASYTABLE_RUN_XTEST=true to enable).\n")
    return(invisible(NULL))
  }

  script <- file.path(pkg_root, "tests", "xtest", "test-api-and-layout.R")
  if (!file.exists(script)) {
    stop("Expected xtest script not found: ", script)
  }
  source(script, local = TRUE)
}

pkg_root <- find_package_root()
cat("Package root:", pkg_root, "\n")

if (identical(profile, "core")) {
  run_core(pkg_root)
} else if (identical(profile, "full")) {
  run_core(pkg_root)
  run_xtest(pkg_root)
} else {
  stop("Unknown profile '", profile, "'. Use 'core' or 'full'.")
}

cat("All requested test layers completed.\n")

#' Check if a package is available
#' @keywords internal
is_package_available <- function(pkg) {
  requireNamespace(pkg, quietly = TRUE)
}

#' Get measure row names
#' @keywords internal
get_measure_names <- function() {
  c("N", "R sq.", "Adj. R sq.", "AIC")
}

#' Get indices of model statistic rows
#' @keywords internal
get_measure_row_indices <- function(table) {
  stat_terms <- attr(table, "stat_terms")

  if (is.null(stat_terms) || length(stat_terms) == 0) {
    stat_terms <- get_measure_names()
  }

  which(table$term %in% stat_terms)
}

#' Get first model statistic row index
#' @keywords internal
get_first_measure_row <- function(table) {
  idx <- get_measure_row_indices(table)
  if (length(idx) == 0) {
    return(NA_integer_)
  }
  min(idx)
}

#' Wrap interaction terms to reduce term-column width
#'
#' For Word output, `"x1 * x2"` becomes `"x1\\n× x2"`.
#'
#' @param terms Character vector of displayed term labels.
#' @param output Output backend used for display formatting. `"word"` uses a
#'   literal multiplication sign, while `"latex"` uses a LaTeX-safe
#'   multiplication symbol.
#'
#' @return Character vector with interaction terms wrapped for display.
#'
#' @keywords internal
wrap_interaction_terms <- function(terms, output = c("word", "latex")) {
  output <- match.arg(output)

  operator <- switch(
    output,
    word = "\u00d7 ",
    latex = "\\(\\times\\) "
  )

  gsub(" * ", paste0("\n", operator), terms, fixed = TRUE)
}

#' Abbreviate variable name
#'
#' Deterministic abbreviation rules:
#' - (Intercept) is preserved
#' - Single-token names truncate to 6 characters
#' - "_" and "." are treated as separators
#' - Two-token names become `token1[1:4] . token2[1:4]`
#' - Three-plus-token names become `t1[1:2] . t2[1:2] . t3[1:2]`
#'
#' @param var_name Character string
#' @param is_intercept Logical indicating if this variable is the intercept
#' @return Character string
#' @keywords internal
abbreviate_var_name <- function(var_name, is_intercept = FALSE) {

  if (is.na(var_name) || !nzchar(var_name)) return(var_name)

  # Preserve intercept label using metadata
  if (is_intercept) return(var_name)

  take <- function(x, k) {
    if (nchar(x) <= k) x else substring(x, 1, k)
  }

  # Treat "_" and "." as separators for composite names
  if (grepl("[_.]", var_name, perl = TRUE)) {

    tokens <- unlist(strsplit(var_name, "[_.]", perl = TRUE))
    tokens <- tokens[tokens != ""]

    if (length(tokens) == 0) return(take(var_name, 6))

    if (length(tokens) == 1) {
      return(take(tokens[1], 6))
    }

    if (length(tokens) == 2) {
      return(paste0(take(tokens[1], 4), ".", take(tokens[2], 4)))
    }

    return(paste0(take(tokens[1], 2), ".", take(tokens[2], 2), ".", take(tokens[3], 2)))
  }

  # Single-token name
  take(var_name, 6)
}

#' Ensure labels are unique via numeric suffix
#' @keywords internal
make_unique_labels <- function(labels) {

  if (!anyDuplicated(labels)) return(labels)

  out <- labels
  tab <- table(labels)
  dup_names <- names(tab[tab > 1])

  for (nm in dup_names) {
    idx <- which(labels == nm)
    if (length(idx) > 1) {
      out[idx[-1]] <- paste0(nm, seq_along(idx)[-1])
    }
  }

  out
}

#' Detect and split factor level suffix using model metadata
#' @keywords internal
split_factor_level <- function(term, levels_map = NULL) {

  if (is.null(levels_map) || length(levels_map) == 0) return(term)
  if (!nzchar(term) || is.na(term)) return(term)
  if (grepl(":", term, fixed = TRUE)) return(term)

  for (var in names(levels_map)) {

    levs <- levels_map[[var]]
    if (is.null(levs) || length(levs) == 0) next

    if (!startsWith(term, var)) next

    suffix <- substring(term, nchar(var) + 1)

    if (suffix %in% levs) {
      return(paste0(var, ":", suffix))
    }
  }

  term
}

#' Format term labels for display
#'
#' Transformations:
#' - Preserve polynomial suffixes (.L, .Q, .C, etc.)
#' - Split factor levels using levels_map
#' - Abbreviate only variable portion (if abbreviate = TRUE)
#' - Ensure uniqueness after abbreviation
#'
#' @param terms Character vector of displayed term labels.
#' @param row_types Optional character vector of row types
#' @param levels_map List mapping factor names to their levels
#' @param abbreviate Logical. Apply abbreviation? Default FALSE
#' @keywords internal
format_term_labels <- function(terms, row_types = NULL, levels_map = NULL, abbreviate = FALSE) {

  formatted <- terms

  # ---- Step 0: Normalize model interaction syntax ----
  # broom encodes interactions as "var1:var2". Convert to "var1 * var2"
  # before splitting factor levels. Factor levels are handled later via
  # split_factor_level() and levels_map (they arrive concatenated, e.g., gendermale).
  formatted <- gsub(":", " * ", formatted, fixed = TRUE)

  # ---- Step 1: Split factor levels ----
  for (i in seq_along(formatted)) {

    term <- formatted[i]

    if (grepl("\\*", term)) {

      parts <- strsplit(term, " \\* ")[[1]]

      for (j in seq_along(parts)) {

        part <- trimws(parts[j])

        if (!grepl(":", part, fixed = TRUE)) {
          parts[j] <- split_factor_level(part, levels_map)
        }
      }

      formatted[i] <- paste(parts, collapse = " * ")

    } else {

      if (!grepl(":", term, fixed = TRUE)) {
        formatted[i] <- split_factor_level(term, levels_map)
      }
    }
  }

  # ---- Step 2: Apply abbreviations (only if requested) ----
  if (isTRUE(abbreviate)) {

    # Helper: abbreviate variable portion only
    abbrev_piece <- function(piece, is_intercept = FALSE) {

      piece <- trimws(piece)

      if (is_intercept) return(piece)

      # Preserve polynomial suffixes like ".L", ".Q", ".C", ".^4"
      if (grepl("\\.[A-Za-z^0-9]+$", piece)) {
        base <- sub("(\\.[A-Za-z^0-9]+)$", "", piece)
        suffix <- sub("^.*(\\.[A-Za-z^0-9]+)$", "\\1", piece)
        return(paste0(abbreviate_var_name(base, is_intercept), suffix))
      }

      if (grepl(":", piece, fixed = TRUE)) {
        parts <- strsplit(piece, ":", fixed = TRUE)[[1]]
        var_name <- parts[1]
        level_part <- paste(parts[-1], collapse = ":")
        # Replace underscores with periods in level part, then abbreviate
        level_part <- gsub("_", ".", level_part, fixed = TRUE)
        level_part <- abbreviate_var_name(level_part, is_intercept)
        return(paste0(abbreviate_var_name(var_name, is_intercept), ":", level_part))
      }

      abbreviate_var_name(piece, is_intercept)
    }

    for (i in seq_along(formatted)) {

      term <- formatted[i]
      is_idx_intercept <- !is.null(row_types) && row_types[i] == "intercept"

      if (grepl("\\*", term)) {

        parts <- strsplit(term, " \\* ")[[1]]
        parts <- vapply(parts, function(p) abbrev_piece(p, is_idx_intercept), character(1))
        formatted[i] <- paste(parts, collapse = " * ")

      } else {

        formatted[i] <- abbrev_piece(term, is_idx_intercept)
      }
    }

    # ---- Step 3: Ensure uniqueness ----
    formatted <- make_unique_labels(formatted)
  }

  formatted
}

build_levels_map <- function(model_list) {
  out <- list()

  for (m in model_list) {
    xl <- m$xlevels
    if (is.null(xl) || length(xl) == 0) next

    for (nm in names(xl)) {
      out[[nm]] <- unique(c(out[[nm]], xl[[nm]]))
    }
  }

  out
}

# Test term label formatting

test_that("format_term_labels handles polynomial suffixes", {
  terms <- c("financial_prudence.Q", "financial_prudence.L", "var.C")

  result <- format_term_labels(terms, abbreviate = TRUE)

  # Polynomial suffixes should be preserved
  expect_true(grepl("\\.Q$", result[1]))
  expect_true(grepl("\\.L$", result[2]))
  expect_true(grepl("\\.C$", result[3]))
})

test_that("format_term_labels separates factor levels", {
  levels_map <- list(
    digital_confidence = c("low", "mid", "high"),
    advisor_confidence = c("low", "mid", "high"),
    occupation = c("no", "yes")
  )
  terms <- c("digital_confidencelow", "advisor_confidencelow", "occupationno")

  result <- format_term_labels(terms, levels_map = levels_map, abbreviate = TRUE)

  expect_equal(result[1], "digi.conf:low")
  expect_equal(result[2], "advi.conf:low")
  expect_equal(result[3], "occupa:no")
})

test_that("format_term_labels converts interaction colons to asterisks", {
  # After polynomial conversion, interactions should use asterisks
  terms <- c("var1.L:var2", "var1:var2")

  result <- format_term_labels(terms)

  # Should have asterisk for interaction
  expect_true(grepl("\\*", result[1]))
  expect_true(grepl("\\*", result[2]))

  # Polynomial suffix should remain
  expect_true(grepl("\\.L", result[1]))
})

test_that("format_term_labels applies abbreviations", {
  terms <- c("financial_prudence", "digital_confidence", "advisor_confidence")

  result <- format_term_labels(terms, abbreviate = TRUE)

  expect_equal(result[1], "fina.prud")
  expect_equal(result[2], "digi.conf")
  expect_equal(result[3], "advi.conf")
})

test_that("format_term_labels handles complex combined terms", {
  levels_map <- list(
    digital_confidence = c("low", "mid", "high")
  )
  terms <- c("financial_prudence.L:digital_confidencelow")

  result <- format_term_labels(terms, levels_map = levels_map, abbreviate = TRUE)

  expect_true(grepl("fina.prud", result[1]))
  expect_true(grepl("digi.conf", result[1]))
  expect_true(grepl("\\.L", result[1]))
  expect_true(grepl(":low", result[1]))
  expect_true(grepl("\\*", result[1]))
})

test_that("format_term_labels doesn't modify measure names", {
  # Measure names should pass through unchanged
  terms <- c("N", "R sq.", "AIC", "Adj. R sq.")

  result <- format_term_labels(terms)

  expect_equal(result, terms)
})

test_that("format_term_labels handles intercept", {
  terms <- c("(Intercept)")

  result <- format_term_labels(terms)

  # Intercept should be unchanged
  expect_equal(result, terms)
})

test_that("format_term_labels handles simple variable names", {
  terms <- c("wt", "hp", "qsec", "flipper_length_mm")

  result <- format_term_labels(terms)

  # Simple names without special patterns should be unchanged
  # unless they match abbreviation patterns
  expect_equal(result[1], "wt")
  expect_equal(result[2], "hp")
  expect_equal(result[3], "qsec")
  # flipper_length_mm is 17 chars, should be abbreviated (but stays same if not in mapping)
  expect_true(nchar(result[4]) <= nchar(terms[4]))
})

test_that("split_factor_level recognizes common levels", {
  levels_map <- list(
    advisor_confidence = c("low", "mid", "high"),
    var = c("yes", "no")
  )
  terms <- c("advisor_confidencelow", "advisor_confidencemid",
             "advisor_confidencehigh", "varyes", "varno")

  result <- vapply(
    terms,
    split_factor_level,
    character(1),
    levels_map = levels_map,
    USE.NAMES = FALSE
  )

  expect_equal(result[1], "advisor_confidence:low")
  expect_equal(result[2], "advisor_confidence:mid")
  expect_equal(result[3], "advisor_confidence:high")
  expect_equal(result[4], "var:yes")
  expect_equal(result[5], "var:no")
})

test_that("abbreviate_var_name handles long names", {
  expect_equal(abbreviate_var_name("financial_prudence"), "fina.prud")
  expect_equal(abbreviate_var_name("digital_confidence"), "digi.conf")
  expect_equal(abbreviate_var_name("advisor_confidence"), "advi.conf")

  # Test generic abbreviation for long unmapped names
  long_name <- "very_long_variable_name"
  result <- abbreviate_var_name(long_name)
  # Should be shorter than original
  expect_true(nchar(result) < nchar(long_name))

  # Short names should be unchanged
  expect_equal(abbreviate_var_name("short"), "short")
  expect_equal(abbreviate_var_name("wt"), "wt")
})

test_that("coefficient cells use two-line estimate + SE format", {
  parsed <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL, abbreviate = FALSE)

  first_cell <- transformed[[2]][1]
  expect_true(grepl("\n", first_cell, fixed = TRUE))
  expect_true(grepl("\\([0-9]", first_cell))
})

test_that("control indicators are placed in the model-stat block", {
  parsed <- parse_models(test_models_lm, robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = "am", abbreviate = FALSE)

  first_stat_row <- get_first_measure_row(transformed)
  expect_true(first_stat_row > 1)
  expect_identical(transformed$term[first_stat_row], "am")
  expect_true(any(transformed$term %in% c("N", "R sq.", "Adj. R sq.")))
})

test_that("latex output keeps exactly one body divider before model-stat rows", {
  skip_if_not_installed("knitr")
  latex <- as.character(easytable(test_m1, test_m2, output = "latex", control.var = "hp"))

  expect_true(grepl("\\\\midrule\nhp &", latex))
  expect_false(grepl("\\\\midrule\n\\\\addlinespace\n", latex))
})

test_that("latex zebra is limited to coefficient rows", {
  skip_if_not_installed("knitr")
  latex <- as.character(easytable(test_m1, test_m2, output = "latex", highlight = TRUE, control.var = "hp"))

  n_line <- regmatches(latex, regexpr("\nN &[^\n]*", latex))
  control_line <- regmatches(latex, regexpr("\nhp &[^\n]*", latex))

  expect_true(length(n_line) == 1 && nzchar(n_line))
  expect_true(length(control_line) == 1 && nzchar(control_line))

  expect_false(grepl("f0f0f0", n_line, fixed = TRUE))
  expect_false(grepl("f0f0f0", control_line, fixed = TRUE))
})

test_that("csv export keeps estimate and SE in the same cell", {
  tmp <- tempfile(fileext = ".csv")
  on.exit(unlink(tmp), add = TRUE)

  invisible(easytable(test_m1, output = "latex", export.csv = tmp))
  csv_lines <- readLines(tmp, warn = FALSE)

  # One line per row in CSV; coefficient cell should include estimate and (SE)
  expect_true(any(grepl("\\(Intercept\\).*\\([0-9]", csv_lines)))
})

test_that("interaction terms wrap onto a multiplication-sign line in word display", {
  m <- lm(mpg ~ wt * hp, data = mtcars)
  parsed <- parse_models(list(Model1 = m), robust.se = FALSE, margins = FALSE)
  transformed <- transform_table(parsed, control.var = NULL, abbreviate = FALSE)
  wrapped <- wrap_interaction_terms(transformed$term)

  expect_true(any(grepl("\n× ", wrapped, fixed = TRUE)))
})

test_that("latex interaction wrapping uses a LaTeX-safe multiplication sign", {
  wrapped <- wrap_interaction_terms("log_gdp * pop", output = "latex")

  expect_identical(wrapped, "log_gdp\n\\(\\times\\) pop")
})

#' Validate model list input
#'
#' @param model_list A named list of models
#' @return Invisible TRUE if valid, otherwise stops with error
#' @keywords internal
validate_model_list <- function(model_list) {
  if (!is.list(model_list)) {
    stop(
      "Please provide models as a list.\n",
      "Example: list(Model 1 = m1, Model 2 = m2)\n",
      "You supplied an object of class: ", class(model_list)[1],
      call. = FALSE
    )
  }

  if (length(model_list) == 0) {
    stop(
      "No models were found.\n",
      "Please pass at least one model object, for example: easytable(m1)",
      call. = FALSE
    )
  }

  if (is.null(names(model_list)) || any(names(model_list) == "")) {
    stop(
      "Each model needs a name before processing.\n",
      "Example: list(Model 1 = m1, Model 2 = m2)",
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' Check if model is supported type
#'
#' @param model A statistical model object
#' @return Logical indicating if model type is supported
#' @keywords internal
is_supported_model <- function(model) {
  inherits(model, c("lm", "glm"))
}

#' Validate all models in list are supported types
#'
#' @param model_list A named list of models
#' @return Invisible TRUE if valid, otherwise stops with error
#' @keywords internal
validate_model_types <- function(model_list) {
  for (i in seq_along(model_list)) {
    model <- model_list[[i]]
    model_name <- names(model_list)[i]

    if (!is_supported_model(model)) {
      stop(
        "Model '", model_name, "' is not supported.\n",
        "Detected class: ", paste(class(model), collapse = ", "), "\n",
        "easytable currently supports: lm, glm.\n",
        "For other model types, please see the modelsummary package:\n",
        "  https://modelsummary.com",
        call. = FALSE
      )
    }
  }

  invisible(TRUE)
}

#' Validate control variables exist in models
#'
#' @param model_list A named list of models
#' @param control.var Character vector of control variable names
#' @return Invisible TRUE if valid, otherwise stops with error
#' @keywords internal
validate_control_vars <- function(model_list, control.var) {
  if (is.null(control.var)) {
    return(invisible(TRUE))
  }

  if (!is.character(control.var)) {
    stop(
      "`control.var` should be a character vector.\n",
      "Example: control.var = c(\"island\", \"species\")\n",
      "Detected type: ", class(control.var)[1],
      call. = FALSE
    )
  }

  # Check at least one model contains the control variables
  for (var in control.var) {
    var_found <- FALSE

    for (i in seq_along(model_list)) {
      model <- model_list[[i]]
      model_terms <- attr(stats::terms(model), "term.labels")

      # Check if variable appears in model (as is, or in transformations)
      if (any(grepl(var, model_terms, fixed = TRUE))) {
        var_found <- TRUE
        break
      }
    }

    if (!var_found) {
      warning(
        "Control variable '", var, "' was not found in the supplied model formulas.\n",
        "This variable will be ignored for this table.",
        call. = FALSE,
        immediate. = TRUE
      )
    }
  }

  invisible(TRUE)
}

#' Validate parameter types
#'
#' @param robust.se Logical for robust standard errors
#' @param margins Logical for marginal effects
#' @param highlight Logical for highlighting
#' @param export.word Character or NULL for Word export path
#' @param export.csv Character or NULL for CSV export path
#' @param output Character string specifying output format
#' @return Invisible TRUE if valid, otherwise stops with error
#' @keywords internal
validate_parameters <- function(robust.se, margins, highlight, export.word, export.csv, output) {
  is_scalar_flag <- function(x) {
    is.logical(x) && length(x) == 1 && !is.na(x)
  }

  if (!is_scalar_flag(robust.se)) {
    stop("`robust.se` must be either TRUE or FALSE.", call. = FALSE)
  }

  if (!is_scalar_flag(margins)) {
    stop("`margins` must be either TRUE or FALSE.", call. = FALSE)
  }

  if (!is_scalar_flag(highlight)) {
    stop("`highlight` must be either TRUE or FALSE.", call. = FALSE)
  }

  if (!is.null(export.word)) {
    if (!is.character(export.word) || length(export.word) != 1 || !nzchar(export.word)) {
      stop("`export.word` must be a single file path string ending in .docx, or NULL.", call. = FALSE)
    }
    if (!grepl("\\.docx$", export.word, ignore.case = TRUE)) {
      stop("`export.word` must end in '.docx'. Example: export.word = \"mytable.docx\".", call. = FALSE)
    }
    if (output != "word") {
      stop("`export.word` is available only when output = \"word\".", call. = FALSE)
    }
  }

  if (!is.null(export.csv)) {
    if (!is.character(export.csv) || length(export.csv) != 1 || !nzchar(export.csv)) {
      stop("`export.csv` must be a single file path string ending in .csv, or NULL.", call. = FALSE)
    }
    if (!grepl("\\.csv$", export.csv, ignore.case = TRUE)) {
      stop("`export.csv` must end in '.csv'. Example: export.csv = \"mytable.csv\".", call. = FALSE)
    }
  }

  invisible(TRUE)
}

#' Validate output format
#'
#' @param output Character string specifying output format
#' @return A normalized output string
#' @keywords internal
validate_output_format <- function(output) {
  if (!is.character(output) || length(output) != 1 || !nzchar(output)) {
    stop(
      "`output` must be one value: \"word\" or \"latex\".",
      call. = FALSE
    )
  }

  output <- tolower(output)
  valid <- c("word", "latex")

  if (!output %in% valid) {
    stop(
      "Unknown output format: '", output, "'.\n",
      "Please use output = \"word\" or output = \"latex\".",
      call. = FALSE
    )
  }

  output
}

#' Check format-specific dependencies
#'
#' @param output Output format ("word" or "latex")
#' @return Invisible TRUE if dependencies available, otherwise stops with error
#' @keywords internal
check_format_dependencies <- function(output) {
  if (output == "word") {
    if (!requireNamespace("flextable", quietly = TRUE)) {
      stop(
        "Word output requires the 'flextable' package.\n",
        "Install it with: install.packages('flextable')",
        call. = FALSE
      )
    }
  }

  if (output == "latex") {
    if (!requireNamespace("knitr", quietly = TRUE)) {
      stop(
        "LaTeX output requires the 'knitr' package.\n",
        "Install it with: install.packages('knitr')",
        call. = FALSE
      )
    }
  }

  invisible(TRUE)
}

#' Check robust SE dependencies
#'
#' @param robust.se Logical indicating if robust SEs are requested
#' @return Invisible TRUE if dependencies available, otherwise stops with error
#' @keywords internal
check_robust_dependencies <- function(robust.se) {
  if (robust.se) {
    missing_pkgs <- c()

    if (!requireNamespace("lmtest", quietly = TRUE)) {
      missing_pkgs <- c(missing_pkgs, "lmtest")
    }

    if (!requireNamespace("sandwich", quietly = TRUE)) {
      missing_pkgs <- c(missing_pkgs, "sandwich")
    }

    if (length(missing_pkgs) > 0) {
      stop(
        "Robust standard errors need additional packages: ",
        paste(missing_pkgs, collapse = ", "), "\n",
        "Install them with: install.packages(c('",
        paste(missing_pkgs, collapse = "', '"), "'))",
        call. = FALSE
      )
    }
  }

  invisible(TRUE)
}

#' Check margins dependencies
#'
#' @param margins Logical indicating if marginal effects are requested
#' @return Invisible TRUE if dependencies available, otherwise stops with error
#' @keywords internal
check_margins_dependencies <- function(margins) {
  if (margins) {
    if (!requireNamespace("margins", quietly = TRUE)) {
      stop(
        "Marginal effects require the 'margins' package.\n",
        "Install it with: install.packages('margins')",
        call. = FALSE
      )
    }
  }

  invisible(TRUE)
}

#' Validate digits parameter
#'
#' @param digits Integer number of digits after the decimal point. Allowed
#'   values are 0 to 4.
#' @return Invisible TRUE if valid, otherwise stops with error
#' @keywords internal
validate_digits <- function(digits) {
  if (!is.numeric(digits) || length(digits) != 1L || is.na(digits) ||
      digits != as.integer(digits) || digits < 0 || digits > 4) {
    stop(
      "`digits` must be a single integer from 0 to 4.\n",
      "  Example: digits = 3",
      call. = FALSE
    )
  }
  invisible(TRUE)
}

#' Validate custom.row parameter
#'
#' @param custom.row Character vector or NULL
#' @param n_models Integer number of models in the table
#' @return Invisible TRUE if valid, otherwise stops with error
#' @keywords internal
validate_custom_row <- function(custom.row, n_models) {
  if (is.null(custom.row)) return(invisible(TRUE))

  if (!is.character(custom.row)) {
    stop(
      "`custom.row` must be a character vector.\n",
      "Example: custom.row = c(\"F. Statistic\", \".004\", \".3\")",
      call. = FALSE
    )
  }

  expected <- n_models + 1L

  if (length(custom.row) != expected) {
    example_vals <- paste(
      c('"F. Statistic"', rep('".00"', n_models)),
      collapse = ", "
    )
    stop(
      sprintf(
        "`custom.row` must have %d element%s (1 row label + %d model value%s), but %d %s provided.\n",
        expected,
        if (expected == 1L) "" else "s",
        n_models,
        if (n_models == 1L) "" else "s",
        length(custom.row),
        if (length(custom.row) == 1L) "was" else "were"
      ),
      sprintf("  Example: custom.row = c(%s)", example_vals),
      call. = FALSE
    )
  }

  invisible(TRUE)
}

#' Validate table_size parameter
#'
#' @param table_size Character string specifying LaTeX table size
#' @param output Character string specifying output format
#' @return Invisible TRUE if valid, otherwise stops with error
#' @keywords internal
validate_table_size <- function(table_size, output) {
  valid_sizes <- c("tiny", "small", "normalsize", "scriptsize")

  if (!is.character(table_size) || length(table_size) != 1 || !nzchar(table_size)) {
    stop("`table_size` must be one value: tiny, small, normalsize, or scriptsize.", call. = FALSE)
  }

  if (!table_size %in% valid_sizes) {
    stop(
      "`table_size` must be one of: ", paste(valid_sizes, collapse = ", "), ".\n",
      "You provided: ", table_size,
      call. = FALSE
    )
  }

  if (table_size != "normalsize" && output != "latex") {
    stop(
      "`table_size` is only used for LaTeX tables.\n",
      "Current output: ", output, "\n",
      "To use table_size, set output = \"latex\".",
      call. = FALSE
    )
  }

  invisible(TRUE)
}

test_that("validate_model_list catches invalid inputs", {
  # Not a list
  expect_error(
    validate_model_list("not a list"),
    "provide models as a list"
  )

  # Unnamed list
  expect_error(
    validate_model_list(list(test_m1, test_m2)),
    "needs a name"
  )

  # Empty list
  expect_error(
    validate_model_list(list()),
    "No models were found"
  )

  # Valid list passes
  expect_invisible(validate_model_list(test_models_lm))
})

test_that("validate_model_types catches unsupported models", {
  # Invalid model type
  bad_model <- structure(list(), class = "unsupported_model")
  expect_error(
    validate_model_types(list(BadModel = bad_model)),
    "not supported"
  )

  # Valid models pass
  expect_invisible(validate_model_types(test_models_lm))
  expect_invisible(validate_model_types(test_models_glm))
})

test_that("validate_control_vars warns about missing variables", {
  # Should warn for non-existent variable
  expect_warning(
    validate_control_vars(test_models_lm, "nonexistent_var"),
    "was not found"
  )

  # Should not warn for existing variable (hp exists in test_m2 and test_m3)
  expect_silent(
    validate_control_vars(test_models_lm, "hp")
  )

  # Should warn for another missing variable
  expect_warning(
    validate_control_vars(test_models_lm, "not_a_var"),
    "was not found"
  )

  # NULL control.var is valid
  expect_invisible(validate_control_vars(test_models_lm, NULL))
})

test_that("validate_parameters checks parameter types", {
  # Invalid robust.se
  expect_error(
    validate_parameters("yes", FALSE, FALSE, NULL, NULL, "word"),
    "robust.se.*TRUE or FALSE"
  )

  # Invalid margins
  expect_error(
    validate_parameters(FALSE, "yes", FALSE, NULL, NULL, "word"),
    "margins.*TRUE or FALSE"
  )

  # Invalid highlight
  expect_error(
    validate_parameters(FALSE, FALSE, 1, NULL, NULL, "word"),
    "highlight.*TRUE or FALSE"
  )

  # Invalid export.csv
  expect_error(
    validate_parameters(FALSE, FALSE, FALSE, NULL, 123, "word"),
    "export.csv.*file path string"
  )

  # Invalid export.word extension
  expect_error(
    validate_parameters(FALSE, FALSE, FALSE, "table.txt", NULL, "word"),
    "must end in '.docx'"
  )

  # export.word only with word output
  expect_error(
    validate_parameters(FALSE, FALSE, FALSE, "table.docx", NULL, "latex"),
    "available only when output = \"word\""
  )

  # Valid parameters pass
  expect_invisible(
    validate_parameters(TRUE, TRUE, TRUE, "table.docx", "table.csv", "word")
  )
})

test_that("check_format_dependencies detects missing packages", {
  # Word format needs flextable
  skip_if_word_tests_unavailable()
  expect_invisible(check_format_dependencies("word"))

  # LaTeX needs knitr
  skip_if_not_installed("knitr")
  expect_invisible(check_format_dependencies("latex"))
})

test_that("check_robust_dependencies detects missing packages", {
  skip_if_not_installed("lmtest")
  skip_if_not_installed("sandwich")
  expect_invisible(check_robust_dependencies(TRUE))

  # FALSE should always pass
  expect_invisible(check_robust_dependencies(FALSE))
})

test_that("check_margins_dependencies detects missing packages", {
  skip_if_not_installed("margins")
  expect_invisible(check_margins_dependencies(TRUE))

  # FALSE should always pass
  expect_invisible(check_margins_dependencies(FALSE))
})

test_that("validate_output_format normalizes and validates output strings", {
  expect_identical(validate_output_format("word"), "word")
  expect_identical(validate_output_format("LaTeX"), "latex")

  expect_error(
    validate_output_format("markdown"),
    "Unknown output format"
  )

  expect_error(
    validate_output_format(c("word", "latex")),
    "must be one value"
  )
})

test_that("validate_custom_row accepts NULL", {
  expect_invisible(validate_custom_row(NULL, 2L))
})

test_that("validate_custom_row accepts correct length vector", {
  expect_invisible(validate_custom_row(c("F. Statistic", ".004", ".3"), 2L))
  expect_invisible(validate_custom_row(c("F. Statistic", ".004"), 1L))
})

test_that("validate_custom_row errors when too few elements", {
  expect_error(
    validate_custom_row(c("F. Statistic", ".004"), 2L),
    "must have 3 elements"
  )
})

test_that("validate_custom_row errors when too many elements", {
  expect_error(
    validate_custom_row(c("F. Statistic", ".004", ".3", ".5"), 2L),
    "must have 3 elements"
  )
})

test_that("validate_custom_row error message contains example", {
  expect_error(
    validate_custom_row(c("F. Statistic"), 2L),
    "Example:"
  )
})

test_that("validate_custom_row errors on non-character input", {
  expect_error(
    validate_custom_row(c(1, 2, 3), 2L),
    "character vector"
  )
})

test_that("validate_digits accepts valid integers", {
  expect_invisible(validate_digits(2))
  expect_invisible(validate_digits(0))
  expect_invisible(validate_digits(4))
})

test_that("validate_digits rejects values outside 0 to 4", {
  expect_error(validate_digits(1.5), "integer from 0 to 4")
  expect_error(validate_digits(-1), "integer from 0 to 4")
  expect_error(validate_digits(5), "integer from 0 to 4")
  expect_error(validate_digits("2"), "integer from 0 to 4")
  expect_error(validate_digits(NA), "integer from 0 to 4")
  expect_error(validate_digits(c(2, 3)), "integer from 0 to 4")
})

#' Format table for LaTeX/PDF output
#'
#' Creates a LaTeX table suitable for PDF documents. Uses booktabs styling
#' and includes significance footnotes via threeparttable package.
#'
#' @param table A transformed data frame from transform_table()
#' @param robust.se Logical indicating if robust standard errors were used
#' @param margins Logical indicating if marginal effects were computed
#' @param highlight Logical indicating whether to highlight significant results
#' @param table_size LaTeX size command: "tiny", "small", "normalsize", "scriptsize"
#'
#' @return A character string containing the LaTeX table code
#' @export
format_latex <- function(table,
                         robust.se = FALSE,
                         margins = FALSE,
                         highlight = FALSE,
                         table_size = "normalsize") {

  # Check that knitr is available
  if (!requireNamespace("knitr", quietly = TRUE)) {
    stop(
      "Package 'knitr' is required for LaTeX output.\n",
      "  Install it with: install.packages('knitr')",
      call. = FALSE
    )
  }

  table <- as.data.frame(table, stringsAsFactors = FALSE)
  table$term <- wrap_interaction_terms(table$term, output = "latex")

  row_types <- table$row_type
  table$row_type <- NULL

  display_headers <- names(table)
  display_headers[1] <- "Coefficient"

  # Identify where model-stat rows start (before LaTeX-safe term normalization)
  first_measure_row <- get_first_measure_row(table)

  insert_measure_divider <- function(result, table, first_measure_row) {
    if (is.na(first_measure_row) || first_measure_row <= 1) {
      return(result)
    }

    first_measure_term <- table$term[first_measure_row]
    lines <- strsplit(result, "\n", fixed = TRUE)[[1]]
    measure_prefix <- paste0(first_measure_term, " &")
    measure_line_idx <- which(startsWith(lines, measure_prefix))[1]

    if (is.na(measure_line_idx)) {
      return(result)
    }

    # Remove default kable spacing row immediately before the measure block.
    if (measure_line_idx > 1 && identical(lines[measure_line_idx - 1], "\\addlinespace")) {
      lines <- lines[-(measure_line_idx - 1)]
      measure_line_idx <- measure_line_idx - 1
    }

    if (measure_line_idx > 1 && identical(lines[measure_line_idx - 1], "\\midrule")) {
      return(paste(lines, collapse = "\n"))
    }

    lines <- append(lines, "\\midrule", after = measure_line_idx - 1)
    paste(lines, collapse = "\n")
  }

  # Replace underscores with periods for LaTeX compatibility
  # Periods don't need escaping and avoid LaTeX special character issues
  for (col in names(table)) {
    if (is.character(table[[col]])) {
      table[[col]] <- gsub("_", ".", table[[col]], fixed = TRUE)
    }
  }

  # Convert newlines in table cells to LaTeX line breaks
  for (j in 1:ncol(table)) {
    for (i in 1:nrow(table)) {
      cell <- table[i, j]
      # Check if cell contains a newline character (from parsing stage)
      if (grepl("\n", cell, fixed = TRUE)) {
        # Replace \n with \\ for LaTeX line break and wrap in shortstack
        cell <- gsub("\n", " \\\\\\\\ ", cell, fixed = TRUE)
        table[i, j] <- paste0("\\shortstack{", cell, "}")
      }
    }
  }

  # Build footnote text
  footnote_text <- "Significance: ***p < .01; **p < .05; *p < .1"

  if (robust.se == TRUE & margins == FALSE) {
    footnote_text <- paste0(footnote_text, ". Note: Robust Standard Errors")
  }

  if (robust.se == FALSE & margins == TRUE) {
    footnote_text <- paste0(footnote_text, ". Note: Average Marginal Effects (AME)")
  }

  if (robust.se == TRUE & margins == TRUE) {
    footnote_text <- paste0(footnote_text, ". Note: Marginal Effects and Robust Standard Errors")
  }

  # Determine coefficient rows and zebra parity (controls excluded from stripe count)
  coef_row_indices <- if (!is.na(first_measure_row) && first_measure_row > 1) {
    seq_len(first_measure_row - 1)
  } else {
    seq_len(nrow(table))
  }

  striped_row <- rep(FALSE, nrow(table))
  stripe_count <- 0
  if (length(coef_row_indices) > 0) {
    for (row_idx in coef_row_indices) {
      is_control <- !is.null(row_types) && row_types[row_idx] == "control"
      if (!is_control) {
        stripe_count <- stripe_count + 1
        striped_row[row_idx] <- (stripe_count %% 2 == 0)
      }
    }
  }

  # Use kableExtra if available for better LaTeX formatting
  if (requireNamespace("kableExtra", quietly = TRUE)) {

    # Apply cell-level styling so zebra and highlight can coexist.
    if (length(coef_row_indices) > 0) {
      for (i in coef_row_indices) {
        is_striped <- striped_row[i]

        # Style term column only for zebra rows
        if (is_striped) {
          table[i, 1] <- kableExtra::cell_spec(
            table[i, 1],
            format = "latex",
            background = "#f0f0f0",
            escape = FALSE
          )
        }

        for (j in 2:ncol(table)) {
          cell_value <- table[i, j]
          is_sig <- isTRUE(highlight) && grepl("\\*", cell_value)

          if (!is_striped && !is_sig) {
            next
          }

          bg_color <- if (is_striped) "#f0f0f0" else "white"

          if (is_sig) {
            is_negative <- grepl("^(\\\\shortstack\\{)?-", cell_value)
            bg_color <- if (is_negative) {
              if (is_striped) "#f0b8b8" else "#ffcccc"
            } else {
              if (is_striped) "#cfeacf" else "#e6ffe6"
            }
          }

          table[i, j] <- kableExtra::cell_spec(
            cell_value,
            format = "latex",
            background = bg_color,
            escape = FALSE
          )
        }
      }
    }

    # Map LaTeX size commands to font sizes (in points)
    size_mapping <- c(
      "tiny" = 5,
      "scriptsize" = 7,
      "small" = 9,
      "normalsize" = 10
    )
    font_size_pt <- size_mapping[table_size]

    kbl <- kableExtra::kbl(
      table,
      format = "latex",
      longtable = TRUE,
      booktabs = TRUE,
      linesep = "",
      escape = FALSE,
      col.names = display_headers,
      align = c("l", rep("c", ncol(table) - 1))
    ) %>%
      kableExtra::kable_styling(
        latex_options = c("repeat_header"),
        font_size = font_size_pt
      )

    # Add footnote
    kbl <- kableExtra::footnote(
      kbl,
      general = footnote_text,
      general_title = "",
      threeparttable = FALSE
    )

    result <- as.character(kbl)

    # Add one clean divider line between coefficient and model-stat blocks
    result <- insert_measure_divider(result, table, first_measure_row)

  } else {
    # Fallback to basic knitr::kable
    result <- knitr::kable(
      table,
      format = "latex",
      longtable = TRUE,
      booktabs = TRUE,
      linesep = "",
      escape = FALSE,
      col.names = display_headers,
      align = c("l", rep("c", ncol(table) - 1))
    )

    # Add one divider line between coefficient and model-stat blocks
    result <- insert_measure_divider(result, table, first_measure_row)

    # Add footnote manually
    result <- paste0(
      result,
      "\n\\\\",
      "\n\\multicolumn{", ncol(table), "}{l}{\\textit{", footnote_text, "}}",
      "\n"
    )

    if (!requireNamespace("kableExtra", quietly = TRUE)) {
      result <- paste0(
        result,
        "\n% Note: Install 'kableExtra' for enhanced LaTeX formatting\n"
      )
    }
  }

  return(knitr::asis_output(result))
}

#' Format table for Word output
#'
#' Creates a flextable object formatted for Microsoft Word documents.
#' Includes significance footnotes, optional highlighting, and notes about
#' robust standard errors or marginal effects.
#'
#' @param table A transformed data frame from transform_table()
#' @param robust.se Logical indicating if robust standard errors were used
#' @param margins Logical indicating if marginal effects were computed
#' @param highlight Logical indicating whether to highlight significant results
#'
#' @return A flextable object ready for export to Word
#' @export
format_word <- function(table,
                        robust.se = FALSE,
                        margins = FALSE,
                        highlight = FALSE) {

  # Check that flextable is available
  if (!requireNamespace("flextable", quietly = TRUE)) {
    stop(
      "Package 'flextable' is required for Word output.\n",
      "  Install it with: install.packages('flextable')",
      call. = FALSE
    )
  }

  table <- as.data.frame(table, stringsAsFactors = FALSE)
  table$term <- wrap_interaction_terms(table$term, output = "word")

  row_types <- table$row_type
  table$row_type <- NULL

  wrap_model_header <- function(x, chunk = 11L) {
    if (is.na(x) || !nzchar(x) || grepl("\\s", x)) {
      return(x)
    }

    if (nchar(x) <= chunk + 1L) {
      return(x)
    }

    starts <- seq.int(1L, nchar(x), by = chunk)
    pieces <- substring(x, starts, pmin(starts + chunk - 1L, nchar(x)))
    paste(pieces, collapse = "\n")
  }

  set_word_widths <- function(ft, n_model_cols) {
    term_width <- if (n_model_cols <= 2) {
      1.91
    } else if (n_model_cols <= 4) {
      1.75
    } else {
      1.62
    }

    model_width <- if (n_model_cols <= 1) {
      1.70
    } else if (n_model_cols <= 2) {
      1.49
    } else if (n_model_cols <= 4) {
      1.30
    } else {
      1.05
    }

    ft <- flextable::width(ft, j = 1, width = term_width)

    if (n_model_cols > 0) {
      ft <- flextable::width(
        ft,
        j = seq.int(2L, n_model_cols + 1L),
        width = model_width
      )
    }

    flextable::set_table_properties(ft, layout = "fixed")
  }

  first_measure_row <- get_first_measure_row(table)
  display_headers <- names(table)
  display_headers[1] <- "Coefficient"
  if (length(display_headers) > 1) {
    display_headers[-1] <- vapply(
      display_headers[-1],
      wrap_model_header,
      character(1),
      USE.NAMES = FALSE
    )
  }

  # Create base flextable
  ft <- flextable::flextable(table) %>%
    flextable::add_footer_lines("Significance: ***p < .01; **p < .05; *p < .1 ")

  ft <- do.call(
    flextable::set_header_labels,
    c(list(x = ft), as.list(stats::setNames(display_headers, names(table))))
  )

  # Add zebra striping for coefficient rows only (skip control rows)
  if (!is.na(first_measure_row) && first_measure_row > 1) {
    stripe_count <- 0
    stripe_rows <- integer(0)

    for (row_idx in 1:(first_measure_row - 1)) {
      is_control <- !is.null(row_types) && row_types[row_idx] == "control"
      if (!is_control) {
        stripe_count <- stripe_count + 1
        if (stripe_count %% 2 == 0) {
          stripe_rows <- c(stripe_rows, row_idx)
        }
      }
    }

    if (length(stripe_rows) > 0) {
      ft <- ft %>%
        flextable::bg(
          i = stripe_rows,
          j = 1:ncol(table),
          part = "body",
          bg = "#f0f0f0"
        )
    }
  }

  # Draw one divider between coefficient and model-stat blocks
  if (!is.na(first_measure_row) && first_measure_row > 1) {
    ft <- ft %>%
      flextable::hline(j = 1:ncol(table), i = first_measure_row - 1, part = "body")
  }

  # Add notes about estimation method
  if (robust.se == TRUE & margins == FALSE) {
    ft <- ft %>%
      flextable::add_footer_lines("Note: Robust Standard Errors")
  }

  if (robust.se == FALSE & margins == TRUE) {
    ft <- ft %>%
      flextable::add_footer_lines("Note: Average Marginal Effects (AME)")
  }

  if (robust.se == TRUE & margins == TRUE) {
    ft <- ft %>%
      flextable::add_footer_lines("Note: Marginal Effects and Robust Standard Errors")
  }

  # Apply highlighting if requested
  if (highlight) {
    coefficient_line <- function(x) sub("\n.*$", "", x)

    # Highlight positive significant coefficients (green)
    for (i in 2:ncol(table)) {
      coef_lines <- coefficient_line(table[[i]])
      p_values <- grepl("\\*", coef_lines)
      ft <- ft %>%
        flextable::bg(
          j = i,
          i = p_values,
          part = "body",
          bg = "#e6ffe6"
        )
    }

    # Highlight negative significant coefficients (red)
    for (i in 2:ncol(table)) {
      coef_lines <- coefficient_line(table[[i]])
      p_values <- grepl("\\*", coef_lines) & grepl("^-", coef_lines)
      ft <- ft %>%
        flextable::bg(
          j = i,
          i = p_values,
          part = "body",
          bg = "#ffcccc"
      )
    }
  }

  ft <- set_word_widths(ft, ncol(table) - 1L)

  return(ft)
}

#' Extract tidy coefficients from a model using base R
#'
#' Includes aliased (perfectly collinear) terms as NA rows, matching the
#' behaviour of broom::tidy().
#'
#' @param model A statistical model object (lm or glm)
#' @return A data frame with columns: term, estimate, std.error, p.value
#' @keywords internal
tidy_model <- function(model) {
  mat   <- summary(model)$coefficients
  p_col <- grep("Pr\\(", colnames(mat), value = TRUE)

  base_df <- data.frame(
    term      = rownames(mat),
    estimate  = mat[, "Estimate"],
    std.error = mat[, "Std. Error"],
    p.value   = mat[, p_col],
    row.names = NULL,
    stringsAsFactors = FALSE
  )

  # summary() drops aliased terms; add them back as NA rows (broom parity)
  all_terms <- names(stats::coef(model))
  aliased   <- setdiff(all_terms, base_df$term)
  if (length(aliased) > 0) {
    na_df <- data.frame(
      term      = aliased,
      estimate  = NA_real_,
      std.error = NA_real_,
      p.value   = NA_real_,
      stringsAsFactors = FALSE
    )
    base_df <- rbind(base_df, na_df)
    base_df <- base_df[match(all_terms, base_df$term), ]
    rownames(base_df) <- NULL
  }

  base_df
}

#' Extract tidy coefficients from a coeftest matrix
#'
#' @param x A coeftest object from lmtest::coeftest()
#' @return A data frame with columns: term, estimate, std.error, p.value
#' @keywords internal
tidy_coeftest <- function(x) {
  mat <- as.matrix(x)
  p_col <- grep("Pr\\(", colnames(mat), value = TRUE)
  data.frame(
    term      = rownames(mat),
    estimate  = mat[, "Estimate"],
    std.error = mat[, "Std. Error"],
    p.value   = mat[, p_col],
    row.names = NULL,
    stringsAsFactors = FALSE
  )
}

#' Extract tidy marginal effects from a margins object
#'
#' @param model A statistical model object (lm or glm)
#' @return A data frame with columns: term, estimate, std.error, p.value
#' @keywords internal
tidy_margins <- function(model) {
  s <- summary(margins::margins(model))
  data.frame(
    term      = s[["factor"]],
    estimate  = s[["AME"]],
    std.error = s[["SE"]],
    p.value   = s[["p"]],
    row.names = NULL,
    stringsAsFactors = FALSE
  )
}

#' Parse a single statistical model
#'
#' Extracts coefficients, standard errors, and p-values from a statistical model.
#' Supports options for robust standard errors and marginal effects.
#'
#' @param model A statistical model object (lm or glm)
#' @param robust.se Logical indicating whether to use robust standard errors
#' @param margins Logical indicating whether to compute marginal effects
#'
#' @return A data frame with columns: term, estimate, std.error, p.value
#' @keywords internal
parse_single_model <- function(model, robust.se = FALSE, margins = FALSE) {

  if (isTRUE(robust.se) && !isTRUE(margins)) {
    m <- tidy_coeftest(
      lmtest::coeftest(model, vcov = sandwich::vcovHC(model, type = "HC"))
    )
  } else if (!isTRUE(robust.se) && isTRUE(margins)) {
    m <- tidy_margins(model)
  } else if (isTRUE(robust.se) && isTRUE(margins)) {
    m1 <- tidy_coeftest(
      lmtest::coeftest(model, vcov = sandwich::vcovHC(model, type = "HC"))
    ) %>%
      dplyr::filter(term != "(Intercept)") %>%
      dplyr::select(-estimate)

    m2 <- tidy_margins(model) %>%
      dplyr::select(term, estimate)

    m <- dplyr::left_join(m1, m2, by = "term")
  } else {
    m <- tidy_model(model)
  }

  m
}

# Internal formatter for coefficient estimates and standard errors.
format_coefficient_value <- function(x, digits = 2) {
  vapply(x, function(value) {
    if (is.nan(value)) {
      return("NaN")
    }

    if (is.na(value)) {
      return(NA_character_)
    }

    if (is.infinite(value)) {
      return(as.character(value))
    }

    use_scientific <- abs(value) >= 10000 || (value != 0 && round(value, digits) == 0)

    if (use_scientific) {
      scientific <- sprintf("%.*E", digits, value)
      parts <- strsplit(scientific, "E", fixed = TRUE)[[1]]
      exponent <- as.integer(parts[2])
      return(paste0(parts[1], "E", exponent))
    }

    sprintf("%.*f", digits, value)
  }, character(1))
}

#' Format coefficients with significance stars and standard errors
#'
#' @param coef_data A data frame with columns: term, estimate, std.error, p.value
#' @param digits Integer number of digits after the decimal point for
#'   coefficients and standard errors, including the mantissa in scientific
#'   notation. Allowed values are 0 to 4. Default 2. Does not affect p-value
#'   star thresholds.
#'
#' @return A data frame with formatted coefficient strings
#' @keywords internal
format_coefficients <- function(coef_data, digits = 2) {
  coef_data %>%
    dplyr::select(term, estimate, std.error, p.value) %>%
    dplyr::mutate(
      significance = dplyr::case_when(
        p.value < 0.01 ~ "***",
        p.value >= 0.01 & p.value <= 0.05 ~ "** ",
        p.value > 0.05 & p.value <= 0.1  ~ "*  ",
        TRUE ~ "   "
      )
    ) %>%
    dplyr::mutate(
      estimate  = format_coefficient_value(estimate, digits),
      std.error = format_coefficient_value(std.error, digits)
    ) %>%
    dplyr::mutate(
      estimate = paste0(
        estimate, " ",
        significance, "\n", "(",
        std.error, ")"
      )
    ) %>%
    dplyr::select(term, estimate)
}

#' Extract goodness-of-fit measures from a model
#'
#' Rounding is fixed per statistic and independent of the user-facing \code{digits}
#' option: N = 0, R sq. = 2, Adj. R sq. = 2, AIC = 0. AIC is only reported
#' for \code{glm} models.
#'
#' @param model A statistical model object (lm or glm)
#'
#' @return A data frame with model fit statistics
#' @keywords internal
extract_model_measures <- function(model) {
  s <- summary(model)

  fmt <- function(x, d) as.character(round(x, d))

  data.frame(
    term = c("N", "R sq.", "Adj. R sq.", "AIC"),
    estimate = c(
      fmt(stats::nobs(model),                                   0L),
      fmt(if (!is.null(s$r.squared))     s$r.squared     else NA, 2L),
      fmt(if (!is.null(s$adj.r.squared)) s$adj.r.squared else NA, 2L),
      fmt(if (!is.null(s$aic))           s$aic           else NA, 0L)
    ),
    stringsAsFactors = FALSE
  )
}

#' Parse a model and return formatted results
#'
#' Main parsing function that combines coefficient extraction, formatting,
#' and measure calculation.
#'
#' @param model A statistical model object (lm or glm)
#' @param robust.se Logical indicating whether to use robust standard errors
#' @param margins Logical indicating whether to compute marginal effects
#' @param digits Integer number of digits after the decimal point for
#'   coefficients and standard errors, including the mantissa in scientific
#'   notation. Allowed values are 0 to 4.
#'
#' @return A data frame with one column for terms and one for formatted estimates
#' @keywords internal
parse_model <- function(model, robust.se = FALSE, margins = FALSE, digits = 2) {

  coef_data <- parse_single_model(model, robust.se, margins)
  mod.df <- format_coefficients(coef_data, digits)
  measures <- extract_model_measures(model)

  dplyr::bind_rows(mod.df, measures)
}

#' Parse multiple models into a combined table
#'
#' @param model_list A named list of statistical models
#' @param robust.se Logical indicating whether to use robust standard errors
#' @param margins Logical indicating whether to compute marginal effects
#' @param digits Integer number of digits after the decimal point for
#'   coefficients and standard errors, including the mantissa in scientific
#'   notation. Allowed values are 0 to 4.
#'
#' @return A data frame with terms in rows and models in columns
#' @keywords internal
parse_models <- function(model_list, robust.se = FALSE, margins = FALSE, digits = 2) {

  model_names <- names(model_list)

  if (is.null(model_names) || any(!nzchar(model_names))) {
    stop("model_list must be a named list of models.", call. = FALSE)
  }

  # Build factor-level map from model metadata for later label splitting
  levels_map <- build_levels_map(model_list)

  # Parse first model
  result_table <- parse_model(model_list[[1]], robust.se, margins, digits)
  names(result_table)[2] <- model_names[1]

  # Parse remaining models
  if (length(model_list) > 1) {
    for (i in 2:length(model_list)) {
      model_table <- parse_model(model_list[[i]], robust.se, margins, digits)
      names(model_table)[2] <- model_names[i]

      result_table <- dplyr::full_join(result_table, model_table, by = "term")
    }
  }

  # Carry levels_map forward without changing return type
  attr(result_table, "levels_map") <- levels_map

  result_table
}

test_that("parse_single_model extracts coefficients from lm", {
  result <- parse_single_model(test_m1, robust.se = FALSE, margins = FALSE)

  expect_s3_class(result, "data.frame")
  expect_true("term" %in% names(result))
  expect_true("estimate" %in% names(result))
  expect_true("std.error" %in% names(result))
  expect_true("p.value" %in% names(result))
})

test_that("parse_single_model works with robust SE", {
  skip_if_not_installed("lmtest")
  skip_if_not_installed("sandwich")

  result <- parse_single_model(test_m1, robust.se = TRUE, margins = FALSE)

  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)
})

test_that("parse_single_model works with margins", {
  skip_if_not_installed("margins")

  result <- parse_single_model(test_m1, robust.se = FALSE, margins = TRUE)

  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)
})

test_that("parse_single_model works with robust SE and margins", {
  skip_if_not_installed("lmtest")
  skip_if_not_installed("sandwich")
  skip_if_not_installed("margins")

  result <- parse_single_model(test_m2, robust.se = TRUE, margins = TRUE)

  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)
  # Should not have intercept when using margins
  expect_false("(Intercept)" %in% result$term)
})

test_that("format_coefficients adds significance stars", {
  coef_data <- data.frame(
    term = c("x1", "x2", "x3", "x4"),
    estimate = c(1.5, 2.0, 3.0, 0.5),
    std.error = c(0.5, 0.6, 0.7, 0.8),
    p.value = c(0.005, 0.03, 0.08, 0.5)
  )

  result <- format_coefficients(coef_data)

  expect_true("estimate" %in% names(result))
  expect_true(grepl("\\*\\*\\*", result$estimate[1])) # p < 0.01
  expect_true(grepl("\\*\\*", result$estimate[2]))    # p < 0.05
  expect_true(grepl("\\*", result$estimate[3]))       # p < 0.1
  expect_false(grepl("\\*", result$estimate[4]))      # p > 0.1
})

test_that("extract_model_measures returns fit statistics", {
  result <- extract_model_measures(test_m1)

  expect_s3_class(result, "data.frame")
  expect_true("term" %in% names(result))
  expect_true("estimate" %in% names(result))
  expect_true("N" %in% result$term)
  expect_true("R sq." %in% result$term)
  expect_true("Adj. R sq." %in% result$term)
  expect_true("AIC" %in% result$term)
})

test_that("parse_model combines coefficients and measures", {
  result <- parse_model(test_m1, robust.se = FALSE, margins = FALSE)

  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 4) # At least intercept, coef, and measures
  expect_true("term" %in% names(result))

  # Check for both coefficients and measures
  expect_true("(Intercept)" %in% result$term)
  expect_true("N" %in% result$term)
})

test_that("parse_models handles multiple models", {
  result <- parse_models(test_models_lm, robust.se = FALSE, margins = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(ncol(result), length(test_models_lm) + 1) # term + 3 models
  expect_equal(names(result)[1], "term")
  expect_equal(names(result)[2:4], names(test_models_lm))
})

test_that("parse_models works with single model", {
  result <- parse_models(test_single_model, robust.se = FALSE, margins = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(ncol(result), 2) # term + 1 model
})

test_that("parse_models handles GLM models", {
  result <- parse_models(test_models_glm, robust.se = FALSE, margins = FALSE)

  expect_s3_class(result, "data.frame")
  expect_equal(ncol(result), length(test_models_glm) + 1)
})

test_that("format_coefficients respects digits parameter", {
  coef_data <- data.frame(
    term      = "x",
    estimate  = 1.23456,
    std.error = 0.56789,
    p.value   = 0.001,
    stringsAsFactors = FALSE
  )

  result2 <- format_coefficients(coef_data, digits = 2)
  expect_true(grepl("1.23", result2$estimate))
  expect_true(grepl("0.57", result2$estimate))

  result4 <- format_coefficients(coef_data, digits = 4)
  expect_true(grepl("1.2346", result4$estimate))
  expect_true(grepl("0.5679", result4$estimate))

  result0 <- format_coefficients(coef_data, digits = 0)
  expect_true(grepl("^1 ", result0$estimate))
})

test_that("format_coefficient_value uses fixed and scientific notation as needed", {
  expect_identical(
    format_coefficient_value(
      c(0.1234, 1.2, 0, 0.0000325, -0.0000325, 11100, -11100, NA, Inf, -Inf),
      digits = 2
    ),
    c("0.12", "1.20", "0.00", "3.25E-5", "-3.25E-5", "1.11E4", "-1.11E4", NA, "Inf", "-Inf")
  )

  expect_identical(
    format_coefficient_value(c(1.2, 0.0000325, 11100), digits = 3),
    c("1.200", "3.250E-5", "1.110E4")
  )
})

test_that("format_coefficients applies formatter to estimates and standard errors", {
  coef_data <- data.frame(
    term = c("tiny", "large", "special"),
    estimate = c(-0.0000325, 11100, Inf),
    std.error = c(0.0000042, 12000, -Inf),
    p.value = c(0.001, 0.2, 0.2),
    stringsAsFactors = FALSE
  )

  result <- format_coefficients(coef_data, digits = 2)

  expect_match(result$estimate[1], "^-3\\.25E-5 \\*\\*\\*")
  expect_match(result$estimate[1], "\\(4\\.20E-6\\)$")
  expect_match(result$estimate[2], "^1\\.11E4")
  expect_match(result$estimate[2], "\\(1\\.20E4\\)$")
  expect_match(result$estimate[3], "^Inf")
  expect_match(result$estimate[3], "\\(-Inf\\)$")
})

test_that("parse_models carries scientific notation into assembled table cells", {
  m <- lm(I(mpg / 1e6) ~ wt, data = mtcars)

  parsed <- parse_models(list(Model1 = m), digits = 2)
  transformed <- transform_table(parsed, control.var = NULL, abbreviate = FALSE)

  wt_cell <- transformed[transformed$term == "wt", "Model1"]
  expect_length(wt_cell, 1)
  expect_true(grepl("E-", wt_cell, fixed = TRUE))
  expect_true(grepl("\n\\(", wt_cell))
})

test_that("extract_model_measures uses fixed rounding per stat", {
  m <- lm(mpg ~ wt + hp, data = mtcars)
  result <- extract_model_measures(m)

  n_val   <- result$estimate[result$term == "N"]
  rsq_val <- result$estimate[result$term == "R sq."]

  # N should be a whole number (no decimal point)
  expect_false(grepl("\\.", n_val))
  # R sq. should have exactly 2 decimal places
  expect_true(grepl("^0\\.[0-9]{2}$", rsq_val))
})

test_that("easytable digits parameter flows through to output", {
  skip_if_word_tests_unavailable()

  result3 <- easytable(test_m1, digits = 3)
  ft_data <- result3$body$dataset
  # At least one cell should contain 3 decimal places
  coef_cells <- ft_data[[2]]
  expect_true(any(grepl("\\.[0-9]{3}", coef_cells)))
})

is_true_env <- function(name, default = "false") {
  tolower(Sys.getenv(name, default)) %in% c("1", "true", "yes", "y")
}

skip_if_word_tests_unavailable <- function() {
  if (is_true_env("EASYTABLE_SKIP_WORD_TESTS")) {
    skip("Word/flextable tests skipped by EASYTABLE_SKIP_WORD_TESTS.")
  }
  skip_if_not_installed("flextable")
}

skip_if_xtest_disabled <- function() {
  if (!is_true_env("EASYTABLE_RUN_XTEST")) {
    skip("Extended sandbox tests are disabled (set EASYTABLE_RUN_XTEST=true).")
  }
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom dplyr bind_rows filter full_join left_join mutate rename select case_when across where .data
#' @importFrom magrittr %>%
#' @importFrom stats nobs
#' @importFrom utils write.csv
## usethis namespace: end
NULL

---
title: "Developer Roadmap: API Clarity and Renderer Stability"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Developer Roadmap: API Clarity and Renderer Stability}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This document is the implementation plan for keeping `easytable` simple for beginners while making internals safer for long-term maintenance.

## Current State

`easytable()` currently does three jobs at once:

1. Parse and transform model results.
2. Render to output-specific objects (`flextable` or LaTeX string).
3. Optionally write files (`export.word`, `export.csv`).

This is convenient, but can be unclear for new users and harder to maintain.

## API Rework Plan (Phased)

### Phase 1: Keep behavior, improve clarity (current target)

1. Keep `easytable()` as main user entry point.
2. Improve parameter validation and friendly errors.
3. Document side effects (`export.word`, `export.csv`) explicitly.

### Phase 2: Introduce explicit workflow functions

Add clearly separated helpers:

1. `easytable_build()`:
   - takes models
   - returns backend-agnostic table spec (display strings + style map)
2. `easytable_render()`:
   - takes spec
   - returns renderer object (`flextable`, LaTeX)
3. `easytable_export()`:
   - takes rendered object/spec
   - writes files

`easytable()` remains a convenience wrapper that calls the three steps.

### Phase 3: Deprecation policy and migration

1. Keep wrapper stable for beginners.
2. Mark low-level transitions with soft deprecation warnings.
3. Provide migration examples in release notes.

## Beginner Navigation Plan

Beginner path should always be:

1. Fit models.
2. Call `easytable(...)`.
3. Optionally add one argument at a time (`highlight`, `control.var`, `output`, exports).

Avoid requiring beginners to understand renderer internals or object classes.

## Renderer Adapter Contract

To reduce drift between Word/HTML and LaTeX:

1. Core transformation must own row semantics:
   - coefficient vs model-stat rows
   - control indicator placement
2. Core must own finalized display strings:
   - estimate + stars + `(SE)` line breaks
3. Renderers must only map style tokens to backend syntax.

## External Dependency Change Strategy (example: flextable)

When dependencies evolve:

1. Pin a compatibility floor in `DESCRIPTION`.
2. Maintain adapter-focused tests in `tests/testthat`.
3. Avoid deep reliance on undocumented object internals.
4. Keep fallback behavior documented (for missing optional packages).

## Compatibility Checklist for Releases

Before each release:

1. Run core tests with `devtools::test()`.
2. Run constrained profile:
   - `EASYTABLE_SKIP_WORD_TESTS=true Rscript -e "devtools::test()"`
3. Spot-check parity on key invariants:
   - two-line coefficient cells
   - zebra only on coefficient rows
   - one coefficient/stat divider

## Design Reference

For design language and contributor guardrails, see `DESIGN_PHILOSOPHY.md` in the repository root.

---
title: "Penguins Tutorial: Easy to Use, Easy to Read"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Penguins Tutorial: Easy to Use, Easy to Read}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

This tutorial shows the full `easytable` workflow using `palmerpenguins`.

## 1) Fit a sequence of models

```{r}
library(easytable)
library(palmerpenguins)

m1 <- lm(body_mass_g ~ flipper_length_mm, data = penguins)
m2 <- lm(body_mass_g ~ flipper_length_mm + species, data = penguins)
m3 <- lm(body_mass_g ~ flipper_length_mm + species + island, data = penguins)
```

## 2) Create a readable baseline table

```{r}
easytable(
  m1, m2, m3,
  model.names = c("Baseline", "With Species", "Full Model")
)
```

By default, each coefficient cell is two lines:

1. Estimate + significance stars
2. Standard error in parentheses

## 3) Highlight significant coefficients

```{r}
easytable(
  m1, m2, m3,
  model.names = c("Baseline", "With Species", "Full Model"),
  highlight = TRUE
)
```

## 4) Collapse controls into indicator rows

```{r}
easytable(
  m1, m2, m3,
  model.names = c("Baseline", "With Species", "Full Model"),
  control.var = "island",
  highlight = TRUE
)
```

`control.var` is useful when models include many factor levels or fixed effects.

## 5) LaTeX output for PDF workflows

```{r}
easytable(
  m1, m2, m3,
  model.names = c("Baseline", "With Species", "Full Model"),
  output = "latex",
  control.var = "island",
  highlight = TRUE
)
```

## 6) Robust standard errors (optional dependency path)

```{r}
if (requireNamespace("lmtest", quietly = TRUE) &&
    requireNamespace("sandwich", quietly = TRUE)) {
  easytable(
    m1, m2, m3,
    model.names = c("Baseline", "With Species", "Full Model"),
    robust.se = TRUE
  )
}
```

## 7) Marginal effects (optional dependency path)

```{r}
if (requireNamespace("margins", quietly = TRUE)) {
  easytable(
    m2, m3,
    model.names = c("With Species", "Full Model"),
    margins = TRUE
  )
}
```

## 8) Export outputs

```{r eval = FALSE}
easytable(
  m1, m2, m3,
  model.names = c("Baseline", "With Species", "Full Model"),
  highlight = TRUE,
  export.word = "penguins_table.docx",
  export.csv = "penguins_table.csv"
)
```

## Design Notes

`easytable` keeps these display invariants:

1. Two-line coefficient cells.
2. Zebra only in coefficient rows.
3. One divider between coefficient rows and model-stat rows.
4. Control indicators in the model-stat block.

These defaults are intentional so tables remain legible in long workflows.

---
title: "easytable: Quick Start"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{easytable: Quick Start}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

`easytable` is built around one promise:

1. Easy to use.
2. Easy to read.

## Minimal Example

```{r}
library(easytable)

m1 <- lm(mpg ~ wt, data = mtcars)
m2 <- lm(mpg ~ wt + hp, data = mtcars)

easytable(m1, m2)
```

## Output Formats

### Word / HTML path (default)

```{r}
easytable(m1, m2, output = "word")
```

### LaTeX / PDF path

```{r}
easytable(m1, m2, output = "latex")
```

## Common Options

```{r}
easytable(
  m1, m2,
  model.names = c("Baseline", "With Controls"),
  highlight = TRUE
)
```

```{r}
easytable(
  m1, m2,
  robust.se = TRUE
)
```

## Export Files

```{r eval = FALSE}
easytable(
  m1, m2,
  export.word = "table.docx",
  export.csv = "table.csv"
)
```

## Next Steps

- Full end-to-end walkthrough: `vignette("penguins-tutorial", package = "easytable")`
- Developer API and architecture plan: `vignette("developer-roadmap", package = "easytable")`

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{make_unique_labels}
\alias{make_unique_labels}
\title{Ensure labels are unique via numeric suffix}
\usage{
make_unique_labels(labels)
}
\description{
Ensure labels are unique via numeric suffix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pipe.R
\name{\%>\%}
\alias{\%>\%}
\title{Pipe Operator}
\usage{
lhs \%>\% rhs
}
\value{
The result of evaluating the right-hand side expression after the
left-hand side has been passed through the pipe.
}
\description{
Re-export of \verb{\%>\%} from 'magrittr'.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{split_factor_level}
\alias{split_factor_level}
\title{Detect and split factor level suffix using model metadata}
\usage{
split_factor_level(term, levels_map = NULL)
}
\description{
Detect and split factor level suffix using model metadata
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{check_margins_dependencies}
\alias{check_margins_dependencies}
\title{Check margins dependencies}
\usage{
check_margins_dependencies(margins)
}
\arguments{
\item{margins}{Logical indicating if marginal effects are requested}
}
\value{
Invisible TRUE if dependencies available, otherwise stops with error
}
\description{
Check margins dependencies
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_parameters}
\alias{validate_parameters}
\title{Validate parameter types}
\usage{
validate_parameters(
  robust.se,
  margins,
  highlight,
  export.word,
  export.csv,
  output
)
}
\arguments{
\item{robust.se}{Logical for robust standard errors}

\item{margins}{Logical for marginal effects}

\item{highlight}{Logical for highlighting}

\item{export.word}{Character or NULL for Word export path}

\item{export.csv}{Character or NULL for CSV export path}

\item{output}{Character string specifying output format}
}
\value{
Invisible TRUE if valid, otherwise stops with error
}
\description{
Validate parameter types
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_table.R
\name{deduplicate_control_vars}
\alias{deduplicate_control_vars}
\title{Remove duplicate control variable rows}
\usage{
deduplicate_control_vars(table)
}
\arguments{
\item{table}{A data frame with regression results}
}
\value{
A data frame with duplicates removed
}
\description{
After collapsing control variables, removes duplicate rows but keeps
one representative row per control variable for each model.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_control_vars}
\alias{validate_control_vars}
\title{Validate control variables exist in models}
\usage{
validate_control_vars(model_list, control.var)
}
\arguments{
\item{model_list}{A named list of models}

\item{control.var}{Character vector of control variable names}
}
\value{
Invisible TRUE if valid, otherwise stops with error
}
\description{
Validate control variables exist in models
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/easytab.R
\name{easytable}
\alias{easytable}
\title{Create Multi-Format Regression Tables}
\usage{
easytable(
  ...,
  model.names = NULL,
  output = "word",
  export.word = NULL,
  export.csv = NULL,
  robust.se = FALSE,
  control.var = NULL,
  margins = FALSE,
  highlight = FALSE,
  abbreviate = FALSE,
  table_size = "normalsize",
  digits = 2,
  custom.row = NULL
)
}
\arguments{
\item{...}{Statistical model objects (lm or glm). Pass models directly
like \code{easytable(m1, m2, m3)}.}

\item{model.names}{Character vector of custom names for model columns.
If NULL (default), columns are named "Model 1", "Model 2", etc.
Length must match number of models.}

\item{output}{Character string specifying output format. One of:
\itemize{
\item \code{"word"} - Microsoft Word via flextable (default)
\item \code{"latex"} - LaTeX for PDF output
}}

\item{export.word}{Character string ending in \code{.docx} for Word file export.
Only supported when \code{output = "word"}. If NULL (default), no file is written.}

\item{export.csv}{Character string ending in \code{.csv} for CSV export.
If NULL (default), no CSV file is written.}

\item{robust.se}{Logical. Use robust standard errors (HC type)? Default FALSE.
Requires packages: lmtest, sandwich}

\item{control.var}{Character vector of variable names to group as "control
variables". These will be collapsed into single rows showing "Y" for
presence instead of individual coefficients. Default NULL.}

\item{margins}{Logical. Compute average marginal effects (AME)? Default FALSE.
Requires package: margins}

\item{highlight}{Logical. Highlight significant coefficients (positive in green,
negative in red)? Default FALSE. Works best with Word output.}

\item{abbreviate}{Logical. Abbreviate variable names for readability? Default FALSE.
When TRUE, long variable names are shortened using deterministic rules.}

\item{table_size}{Character string specifying LaTeX table size. Only works with
\code{output = "latex"}. Options: "tiny", "small", "normalsize", "scriptsize".
Default "normalsize". Error if used with Word output.}

\item{digits}{Number of digits after the decimal point for coefficients and
standard errors, including the mantissa in scientific notation. Allowed
values are 0 to 4. Default 2.}

\item{custom.row}{Optional character vector for an additional row placed at
the bottom of the statistics block. The first element is the row label and
each subsequent element is the value for the corresponding model column.
The vector must have exactly one more element than the number of models.
Default NULL (no extra row).}
}
\value{
Depends on \code{output}:
\itemize{
\item \code{"word"} - A flextable object
\item \code{"latex"} - Character string with LaTeX table code
}
}
\description{
Takes model objects as arguments and creates formatted tables for
Word or LaTeX/PDF output. Supports robust standard errors,
marginal effects, and control variable grouping.
}
\details{
The function extracts coefficients, standard errors, and p-values from each
model, adds significance stars (*** p<.01, ** p<.05, * p<.1), and includes
model fit statistics such as N, R-squared, Adjusted R-squared, and
AIC (for \code{glm} models).

Control variables can be grouped to show presence/absence rather than
individual coefficients for each factor level or transformation.

Term labels are automatically formatted for readability:
\itemize{
\item Factor levels separated with colon (e.g., \code{advisor_confidence:low})
\item Interactions shown with a multiplication sign (e.g., \code{var1 × var2})
\item Polynomial contrasts as L indices (e.g., \code{var:L1}, \code{var:L2})
\item Long variable names abbreviated for clarity
}
}
\section{Dependencies}{

\itemize{
\item Always required: dplyr
\item Word output: flextable
\item LaTeX output: knitr, optionally kableExtra for enhanced formatting
\item Robust SE: lmtest, sandwich
\item Marginal effects: margins
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{is_supported_model}
\alias{is_supported_model}
\title{Check if model is supported type}
\usage{
is_supported_model(model)
}
\arguments{
\item{model}{A statistical model object}
}
\value{
Logical indicating if model type is supported
}
\description{
Check if model is supported type
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_table.R
\name{mark_control_vars}
\alias{mark_control_vars}
\title{Mark control variables with 'Y' indicator}
\usage{
mark_control_vars(table, control.var)
}
\arguments{
\item{table}{A data frame with regression results}

\item{control.var}{Character vector of control variable names}
}
\value{
A data frame with control variables marked as "Y"
}
\description{
Replaces coefficient values with "Y" for control variables to indicate
their presence in the model without showing individual coefficients.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_measure_names}
\alias{get_measure_names}
\title{Get measure row names}
\usage{
get_measure_names()
}
\description{
Get measure row names
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/easytable-package.R
\docType{package}
\name{easytable-package}
\alias{easytable-package}
\title{easytable: Create Multi-Format Regression Tables}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Create publication-ready regression tables in multiple formats, including 'Word', 'HTML', 'LaTeX', and 'PDF', from statistical models. Supports lm() and glm() models. Includes options for marginal effects, control variable grouping, and robust standard errors using methods described in Zeileis (2004) \doi{10.18637/jss.v011.i10}. Tables can be exported to 'Word' via 'flextable' or to 'LaTeX' for 'PDF' output.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/alfredo-hs/easytable}
  \item Report bugs at \url{https://github.com/alfredo-hs/easytable/issues}
}

}
\author{
\strong{Maintainer}: Alfredo Hernandez Sanchez \email{alhdzsz@gmail.com}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_digits}
\alias{validate_digits}
\title{Validate digits parameter}
\usage{
validate_digits(digits)
}
\arguments{
\item{digits}{Integer number of digits after the decimal point. Allowed
values are 0 to 4.}
}
\value{
Invisible TRUE if valid, otherwise stops with error
}
\description{
Validate digits parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{is_package_available}
\alias{is_package_available}
\title{Check if a package is available}
\usage{
is_package_available(pkg)
}
\description{
Check if a package is available
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{wrap_interaction_terms}
\alias{wrap_interaction_terms}
\title{Wrap interaction terms to reduce term-column width}
\usage{
wrap_interaction_terms(terms, output = c("word", "latex"))
}
\arguments{
\item{terms}{Character vector of displayed term labels.}

\item{output}{Output backend used for display formatting. \code{"word"} uses a
literal multiplication sign, while \code{"latex"} uses a LaTeX-safe
multiplication symbol.}
}
\value{
Character vector with interaction terms wrapped for display.
}
\description{
For Word output, \code{"x1 * x2"} becomes \code{"x1\\\\n× x2"}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_models.R
\name{parse_single_model}
\alias{parse_single_model}
\title{Parse a single statistical model}
\usage{
parse_single_model(model, robust.se = FALSE, margins = FALSE)
}
\arguments{
\item{model}{A statistical model object (lm or glm)}

\item{robust.se}{Logical indicating whether to use robust standard errors}

\item{margins}{Logical indicating whether to compute marginal effects}
}
\value{
A data frame with columns: term, estimate, std.error, p.value
}
\description{
Extracts coefficients, standard errors, and p-values from a statistical model.
Supports options for robust standard errors and marginal effects.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_model_types}
\alias{validate_model_types}
\title{Validate all models in list are supported types}
\usage{
validate_model_types(model_list)
}
\arguments{
\item{model_list}{A named list of models}
}
\value{
Invisible TRUE if valid, otherwise stops with error
}
\description{
Validate all models in list are supported types
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_latex.R
\name{format_latex}
\alias{format_latex}
\title{Format table for LaTeX/PDF output}
\usage{
format_latex(
  table,
  robust.se = FALSE,
  margins = FALSE,
  highlight = FALSE,
  table_size = "normalsize"
)
}
\arguments{
\item{table}{A transformed data frame from transform_table()}

\item{robust.se}{Logical indicating if robust standard errors were used}

\item{margins}{Logical indicating if marginal effects were computed}

\item{highlight}{Logical indicating whether to highlight significant results}

\item{table_size}{LaTeX size command: "tiny", "small", "normalsize", "scriptsize"}
}
\value{
A character string containing the LaTeX table code
}
\description{
Creates a LaTeX table suitable for PDF documents. Uses booktabs styling
and includes significance footnotes via threeparttable package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_table.R
\name{transform_table}
\alias{transform_table}
\title{Transform parsed model table}
\usage{
transform_table(parsed_table, control.var = NULL, abbreviate = FALSE)
}
\arguments{
\item{parsed_table}{A data frame from parse_models()}

\item{control.var}{Character vector of control variable names}

\item{abbreviate}{Logical. Abbreviate variable names? Default FALSE}
}
\value{
A transformed data frame ready for formatting
}
\description{
Main transformation function that handles control variables, sorting,
deduplication, organization, and term label formatting.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_output_format}
\alias{validate_output_format}
\title{Validate output format}
\usage{
validate_output_format(output)
}
\arguments{
\item{output}{Character string specifying output format}
}
\value{
A normalized output string
}
\description{
Validate output format
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_custom_row}
\alias{validate_custom_row}
\title{Validate custom.row parameter}
\usage{
validate_custom_row(custom.row, n_models)
}
\arguments{
\item{custom.row}{Character vector or NULL}

\item{n_models}{Integer number of models in the table}
}
\value{
Invisible TRUE if valid, otherwise stops with error
}
\description{
Validate custom.row parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_models.R
\name{tidy_model}
\alias{tidy_model}
\title{Extract tidy coefficients from a model using base R}
\usage{
tidy_model(model)
}
\arguments{
\item{model}{A statistical model object (lm or glm)}
}
\value{
A data frame with columns: term, estimate, std.error, p.value
}
\description{
Includes aliased (perfectly collinear) terms as NA rows, matching the
behaviour of broom::tidy().
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_models.R
\name{parse_models}
\alias{parse_models}
\title{Parse multiple models into a combined table}
\usage{
parse_models(model_list, robust.se = FALSE, margins = FALSE, digits = 2)
}
\arguments{
\item{model_list}{A named list of statistical models}

\item{robust.se}{Logical indicating whether to use robust standard errors}

\item{margins}{Logical indicating whether to compute marginal effects}

\item{digits}{Integer number of digits after the decimal point for
coefficients and standard errors, including the mantissa in scientific
notation. Allowed values are 0 to 4.}
}
\value{
A data frame with terms in rows and models in columns
}
\description{
Parse multiple models into a combined table
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_model_list}
\alias{validate_model_list}
\title{Validate model list input}
\usage{
validate_model_list(model_list)
}
\arguments{
\item{model_list}{A named list of models}
}
\value{
Invisible TRUE if valid, otherwise stops with error
}
\description{
Validate model list input
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_table.R
\name{collapse_control_vars}
\alias{collapse_control_vars}
\title{Collapse control variable rows}
\usage{
collapse_control_vars(table, control.var, levels_map = NULL)
}
\arguments{
\item{table}{A data frame with regression results}

\item{control.var}{Character vector of control variable names to collapse}
}
\value{
A data frame with collapsed control variables
}
\description{
Collapses multiple rows for control variables (including factor levels and
transformations) into a single row per control variable.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_table.R
\name{separate_measures}
\alias{separate_measures}
\title{Separate model statistics from coefficients}
\usage{
separate_measures(table, control.var = NULL)
}
\arguments{
\item{table}{A data frame with regression results}

\item{control.var}{Character vector of control variable names}
}
\value{
A data frame with measures at the bottom
}
\description{
Splits the table into coefficient rows and model-stat rows (control indicators,
N, R sq., etc.), removes empty stat rows, then recombines with stats at the bottom.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{validate_table_size}
\alias{validate_table_size}
\title{Validate table_size parameter}
\usage{
validate_table_size(table_size, output)
}
\arguments{
\item{table_size}{Character string specifying LaTeX table size}

\item{output}{Character string specifying output format}
}
\value{
Invisible TRUE if valid, otherwise stops with error
}
\description{
Validate table_size parameter
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_first_measure_row}
\alias{get_first_measure_row}
\title{Get first model statistic row index}
\usage{
get_first_measure_row(table)
}
\description{
Get first model statistic row index
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{check_format_dependencies}
\alias{check_format_dependencies}
\title{Check format-specific dependencies}
\usage{
check_format_dependencies(output)
}
\arguments{
\item{output}{Output format ("word" or "latex")}
}
\value{
Invisible TRUE if dependencies available, otherwise stops with error
}
\description{
Check format-specific dependencies
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_models.R
\name{format_coefficients}
\alias{format_coefficients}
\title{Format coefficients with significance stars and standard errors}
\usage{
format_coefficients(coef_data, digits = 2)
}
\arguments{
\item{coef_data}{A data frame with columns: term, estimate, std.error, p.value}

\item{digits}{Integer number of digits after the decimal point for
coefficients and standard errors, including the mantissa in scientific
notation. Allowed values are 0 to 4. Default 2. Does not affect p-value
star thresholds.}
}
\value{
A data frame with formatted coefficient strings
}
\description{
Format coefficients with significance stars and standard errors
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{abbreviate_var_name}
\alias{abbreviate_var_name}
\title{Abbreviate variable name}
\usage{
abbreviate_var_name(var_name, is_intercept = FALSE)
}
\arguments{
\item{var_name}{Character string}

\item{is_intercept}{Logical indicating if this variable is the intercept}
}
\value{
Character string
}
\description{
Deterministic abbreviation rules:
\itemize{
\item (Intercept) is preserved
\item Single-token names truncate to 6 characters
\item "_" and "." are treated as separators
\item Two-token names become \verb{token1[1:4] . token2[1:4]}
\item Three-plus-token names become \verb{t1[1:2] . t2[1:2] . t3[1:2]}
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_models.R
\name{extract_model_measures}
\alias{extract_model_measures}
\title{Extract goodness-of-fit measures from a model}
\usage{
extract_model_measures(model)
}
\arguments{
\item{model}{A statistical model object (lm or glm)}
}
\value{
A data frame with model fit statistics
}
\description{
Rounding is fixed per statistic and independent of the user-facing \code{digits}
option: N = 0, R sq. = 2, Adj. R sq. = 2, AIC = 0. AIC is only reported
for \code{glm} models.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{format_term_labels}
\alias{format_term_labels}
\title{Format term labels for display}
\usage{
format_term_labels(
  terms,
  row_types = NULL,
  levels_map = NULL,
  abbreviate = FALSE
)
}
\arguments{
\item{terms}{Character vector of displayed term labels.}

\item{row_types}{Optional character vector of row types}

\item{levels_map}{List mapping factor names to their levels}

\item{abbreviate}{Logical. Apply abbreviation? Default FALSE}
}
\description{
Transformations:
\itemize{
\item Preserve polynomial suffixes (.L, .Q, .C, etc.)
\item Split factor levels using levels_map
\item Abbreviate only variable portion (if abbreviate = TRUE)
\item Ensure uniqueness after abbreviation
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_models.R
\name{parse_model}
\alias{parse_model}
\title{Parse a model and return formatted results}
\usage{
parse_model(model, robust.se = FALSE, margins = FALSE, digits = 2)
}
\arguments{
\item{model}{A statistical model object (lm or glm)}

\item{robust.se}{Logical indicating whether to use robust standard errors}

\item{margins}{Logical indicating whether to compute marginal effects}

\item{digits}{Integer number of digits after the decimal point for
coefficients and standard errors, including the mantissa in scientific
notation. Allowed values are 0 to 4.}
}
\value{
A data frame with one column for terms and one for formatted estimates
}
\description{
Main parsing function that combines coefficient extraction, formatting,
and measure calculation.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/format_word.R
\name{format_word}
\alias{format_word}
\title{Format table for Word output}
\usage{
format_word(table, robust.se = FALSE, margins = FALSE, highlight = FALSE)
}
\arguments{
\item{table}{A transformed data frame from transform_table()}

\item{robust.se}{Logical indicating if robust standard errors were used}

\item{margins}{Logical indicating if marginal effects were computed}

\item{highlight}{Logical indicating whether to highlight significant results}
}
\value{
A flextable object ready for export to Word
}
\description{
Creates a flextable object formatted for Microsoft Word documents.
Includes significance footnotes, optional highlighting, and notes about
robust standard errors or marginal effects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validators.R
\name{check_robust_dependencies}
\alias{check_robust_dependencies}
\title{Check robust SE dependencies}
\usage{
check_robust_dependencies(robust.se)
}
\arguments{
\item{robust.se}{Logical indicating if robust SEs are requested}
}
\value{
Invisible TRUE if dependencies available, otherwise stops with error
}
\description{
Check robust SE dependencies
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_measure_row_indices}
\alias{get_measure_row_indices}
\title{Get indices of model statistic rows}
\usage{
get_measure_row_indices(table)
}
\description{
Get indices of model statistic rows
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_models.R
\name{tidy_coeftest}
\alias{tidy_coeftest}
\title{Extract tidy coefficients from a coeftest matrix}
\usage{
tidy_coeftest(x)
}
\arguments{
\item{x}{A coeftest object from lmtest::coeftest()}
}
\value{
A data frame with columns: term, estimate, std.error, p.value
}
\description{
Extract tidy coefficients from a coeftest matrix
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_table.R
\name{sort_table}
\alias{sort_table}
\title{Sort table with control variables last}
\usage{
sort_table(table)
}
\arguments{
\item{table}{A data frame with regression results}
}
\value{
A sorted data frame
}
\description{
Sorts the table so that control variables (marked with "Y") appear
after regular variables, ordered by how many models they appear in.
}
\keyword{internal}
